import numpy as np

try:
    from utils import *
except ImportError:
    from ORForise.utils import *


class comparator:  # Class to hold global-type variables
    def __init__(self, perfect_Starts=0, perfect_Stops=0, genome_Seq='',
                 genome_Seq_Rev='',
                 genome_Size=0, correct_Frame_Number=0, extended_Start=0,
                 extended_Stop=0, extended_CDS=0, perfect_Matches=collections.OrderedDict(),
                 matched_ORFs=collections.OrderedDict(), multi_Matched_ORFs=collections.defaultdict(list),
                 unmatched_ORFs=collections.OrderedDict(), genes_Detected=collections.OrderedDict(),
                 genes_Undetected=collections.OrderedDict(),
                 out_Of_Frame_ORFs=collections.OrderedDict(), start_Difference=[], stop_Difference=[],
                 orf_Lengths=[], gene_Lengths=[], gene_Pos_Olap=[], gene_Neg_Olap=[], orf_Pos_Olap=[], orf_Neg_Olap=[],
                 m_ORF_Pos_Olap=[], m_ORF_Neg_Olap=[], gene_GC=[],
                 orf_GC=[], m_ORF_GC=[], gene_Short=[], orf_Short=[], m_ORF_Short=[], pos_Strand=0, neg_Strand=0,
                 partial_Hits=collections.OrderedDict()):
        self.perfect_Starts, self.perfect_Stops, self.genome_Seq, self.genome_Seq_Rev, self.genome_Size, self.correct_Frame_Number, self.extended_Start, self.extended_Stop, self.extended_CDS, \
        self.perfect_Matches, self.matched_ORFs, self.multi_Matched_ORFs, self.unmatched_ORFs, self.genes_Detected, self.genes_Undetected, self.out_Of_Frame_ORFs, self.start_Difference, \
        self.stop_Difference, self.orf_Lengths, self.gene_Lengths, self.gene_Pos_Olap, \
        self.gene_Neg_Olap, self.orf_Pos_Olap, self.orf_Neg_Olap, self.m_ORF_Pos_Olap, self.m_ORF_Neg_Olap, self.gene_GC, self.orf_GC, self.m_ORF_GC, self.gene_Short, self.orf_Short, self.m_ORF_Short, self.pos_Strand, \
        self.neg_Strand, self.partial_Hits = perfect_Starts, perfect_Stops, genome_Seq, genome_Seq_Rev, \
                                             genome_Size, correct_Frame_Number, extended_Start, extended_Stop, extended_CDS, perfect_Matches, matched_ORFs, multi_Matched_ORFs, unmatched_ORFs, genes_Detected, genes_Undetected, out_Of_Frame_ORFs, start_Difference, stop_Difference, orf_Lengths, \
                                             gene_Lengths, gene_Pos_Olap, gene_Neg_Olap, orf_Pos_Olap, orf_Neg_Olap, m_ORF_Pos_Olap, m_ORF_Neg_Olap, gene_GC, orf_GC, m_ORF_GC, gene_Short, orf_Short, m_ORF_Short, pos_Strand, neg_Strand, partial_Hits


comp = comparator()


# Not needed
# def keyshift(dictionary, key, diff):
#     if key in dictionary:
#         token = object()
#         keys = [token]*(diff*-1) + dictionary + [token]*diff
#         newkey = keys[keys.index(key)+diff]
#         if newkey is token:
#             print (None)
#         else:
#             to_return = dictionary[newkey].split(',')
#             to_return = to_return[0]+'_'+to_return[1]+'_'+to_return[2]
#             return to_return
#     else:
#         print ('Key not found')


def nuc_Count(start, stop, strand):  # Gets correct seq then returns GC
    if strand == '-':
        r_Start = comp.genome_Size - stop
        r_Stop = comp.genome_Size - start
        seq = (comp.genome_Seq_Rev[r_Start:r_Stop + 1])
    elif strand == '+':
        seq = (comp.genome_Seq[start - 1:stop])
    c = 0
    a = 0
    g = 0
    t = 0
    n = 0
    for i in seq:
        if "C" in i:
            c += 1
        elif "G" in i:
            g += 1
        elif "A" in i:
            a += 1
        elif "T" in i:
            t += 1
        elif "N" in i:
            n += 1
    gc_content = (g + c) * 100 / (a + t + g + c + n)
    return gc_content


def orf_Unmatched(o_Start, o_Stop, o_Strand):
    if o_Strand == '-':
        r_Start = comp.genome_Size - o_Stop
        r_Stop = comp.genome_Size - o_Start
        Unmatched_ORF = str(o_Start) + ',' + str(o_Stop) + ',' + o_Strand + ',' + comp.genome_Seq_Rev[
                                                                                  r_Start:r_Start + 3] + ',' + comp.genome_Seq_Rev[
                                                                                                               r_Stop - 2:r_Stop + 1]
        seq = (comp.genome_Seq_Rev[r_Start:r_Stop + 1])
        comp.unmatched_ORFs.update({Unmatched_ORF: seq})
    elif o_Strand == '+':
        Unmatched_ORF = str(o_Start) + ',' + str(o_Stop) + ',' + o_Strand + ',' + comp.genome_Seq[
                                                                                  o_Start - 1:o_Start + 2] + ',' + comp.genome_Seq[
                                                                                                                   o_Stop - 3:o_Stop]
        seq = (comp.genome_Seq[o_Start - 1:o_Stop])
        comp.unmatched_ORFs.update({Unmatched_ORF: seq})


def genes_Unmatched(g_Start, g_Stop, g_Strand):
    if g_Strand == '-':
        r_Start = comp.genome_Size - g_Stop
        r_Stop = comp.genome_Size - g_Start
        missed_Gene = str(g_Start) + ',' + str(g_Stop) + ',' + g_Strand + ',' + comp.genome_Seq_Rev[
                                                                                r_Start:r_Start + 3] + ',' + comp.genome_Seq_Rev[
                                                                                                             r_Stop - 2:r_Stop + 1]
        genSeq = (comp.genome_Seq_Rev[r_Start:r_Stop + 1])
        comp.genes_Undetected.update({missed_Gene: genSeq})
    elif g_Strand == '+':
        missed_Gene = str(g_Start) + ',' + str(g_Stop) + ',' + g_Strand + ',' + comp.genome_Seq[
                                                                                g_Start - 1:g_Start + 2] + ',' + comp.genome_Seq[
                                                                                                                 g_Stop - 3:g_Stop]
        genSeq = (comp.genome_Seq[g_Start - 1:g_Stop])
        comp.genes_Undetected.update({missed_Gene: genSeq})


def perfect_Matched_Genes(g_Start, g_Stop, g_Strand):
    if g_Strand == '-':
        r_Start = comp.genome_Size - g_Stop
        r_Stop = comp.genome_Size - g_Start
        perfect_Matched_Gene = str(g_Start) + ',' + str(g_Stop) + ',' + g_Strand + ',' + comp.genome_Seq_Rev[
                                                                                         r_Start:r_Start + 3] + ',' + comp.genome_Seq_Rev[
                                                                                                                      r_Stop - 2:r_Stop + 1]
        genSeq = (comp.genome_Seq_Rev[r_Start:r_Stop + 1])
        comp.perfect_Matches.update({perfect_Matched_Gene: genSeq})
    elif g_Strand == '+':
        perfect_Matched_Gene = str(g_Start) + ',' + str(g_Stop) + ',' + g_Strand + ',' + comp.genome_Seq[
                                                                                         g_Start - 1:g_Start + 2] + ',' + comp.genome_Seq[
                                                                                                                          g_Stop - 3:g_Stop]
        genSeq = (comp.genome_Seq[g_Start - 1:g_Stop])
        comp.perfect_Matches.update({perfect_Matched_Gene: genSeq})


def match_Statistics(o_Start, o_Stop, g_Start, g_Stop, g_Strand):
    comp.correct_Frame_Number += 1
    ############ Calculate prediction precision
    if '+' in g_Strand:
        comp.start_Difference.append(o_Start - g_Start)
        comp.stop_Difference.append(o_Stop - g_Stop)
        if g_Start == o_Start:
            comp.perfect_Starts += 1
        if g_Stop == o_Stop:
            comp.perfect_Stops += 1
        if o_Start < g_Start and o_Stop > g_Stop:
            comp.extended_CDS += 1
        if o_Start < g_Start:
            comp.extended_Start += 1
        if o_Stop > g_Stop:
            comp.extended_Stop += 1
    elif '-' in g_Strand:  # Negative strand genes are reversed
        comp.start_Difference.append(o_Stop - g_Stop)
        comp.stop_Difference.append(o_Start - g_Start)
        if g_Start == o_Start:
            comp.perfect_Stops += 1
        if g_Stop == o_Stop:
            comp.perfect_Starts += 1
        if o_Start < g_Start and o_Stop > g_Stop:
            comp.extended_CDS += 1
        if o_Start < g_Start:
            comp.extended_Stop += 1
        if o_Stop > g_Stop:
            comp.extended_Start += 1


def start_Codon_Count(orfs):
    atg, gtg, ttg, att, ctg, other = 0, 0, 0, 0, 0, 0
    other_Starts = []
    for orf in orfs.values():
        codon = orf[1]
        if codon == 'ATG':
            atg += 1
        elif codon == 'GTG':
            gtg += 1
        elif codon == 'TTG':
            ttg += 1
        elif codon == 'ATT':
            att += 1
        elif codon == 'CTG':
            ctg += 1
        else:
            other += 1
            other_Starts.append(codon)
    atg_P = format(100 * atg / len(orfs), '.2f')
    gtg_P = format(100 * gtg / len(orfs), '.2f')
    ttg_P = format(100 * ttg / len(orfs), '.2f')
    att_P = format(100 * att / len(orfs), '.2f')
    ctg_P = format(100 * ctg / len(orfs), '.2f')
    other_Start_P = format(100 * other / len(orfs), '.2f')
    return atg_P, gtg_P, ttg_P, att_P, ctg_P, other_Start_P, other_Starts


def stop_Codon_Count(orfs):
    tag, taa, tga, other = 0, 0, 0, 0
    other_Stops = []
    for orf in orfs.values():
        codon = orf[2]
        if codon == 'TAG':
            tag += 1
        elif codon == 'TAA':
            taa += 1
        elif codon == 'TGA':
            tga += 1
        else:
            other += 1
            other_Stops.append(codon)
    tag_p = format(100 * tag / len(orfs), '.2f')
    taa_p = format(100 * taa / len(orfs), '.2f')
    tga_p = format(100 * tga / len(orfs), '.2f')
    other_Stop_P = format(100 * other / len(orfs), '.2f')
    return tag_p, taa_p, tga_p, other_Stop_P, other_Stops


def candidate_ORF_Selection(gene_Set,
                            candidate_ORFs):  # Select ORF from candidates which is most similar to partially detected gene
    current_Coverage = 0
    candidate_ORF_Difference = 0
    pos = ''
    orf_Details = []
    for c_Pos, c_ORF_Details in candidate_ORFs.items():
        o_Start = int(c_Pos.split(',')[0])
        o_Stop = int(c_Pos.split(',')[1])
        coverage = c_ORF_Details[3]
        orf_Set = set(range(o_Start, o_Stop + 1))
        if coverage > current_Coverage:
            current_Coverage = coverage
            # Return set of elements outside the two sets/DNA ranges
            candidate_ORF_Difference = orf_Set.symmetric_difference(gene_Set)
            pos = c_Pos
            orf_Details = c_ORF_Details
        elif coverage == current_Coverage:
            current_ORF_Difference = orf_Set.symmetric_difference(
                gene_Set)  # Pick least different ORF set from the Gene Set
            if len(current_ORF_Difference) > len(candidate_ORF_Difference):
                pos = c_Pos
                orf_Details = c_ORF_Details
        else:
            print("Match filtered out")
    return pos, orf_Details


def partial_Hit_Calc(g_Start, g_Stop, g_Strand, o_Start, o_Stop):
    if g_Strand == '-':
        r_G_Start = comp.genome_Size - g_Stop
        r_G_Stop = comp.genome_Size - g_Start
        r_O_Start = comp.genome_Size - o_Stop
        r_O_Stop = comp.genome_Size - o_Start
        partial = "Gene:" + str(g_Start) + '_' + str(g_Stop) + '_' + g_Strand + '_' + comp.genome_Seq_Rev[
                                                                                      r_G_Start:r_G_Start + 3] + '_' + comp.genome_Seq_Rev[
                                                                                                                       r_G_Stop - 2:r_G_Stop + 1] + ';Predicted_CDS:' + str(
            o_Start) + '_' + str(o_Stop) + '_' + g_Strand + '_' + comp.genome_Seq_Rev[
                                                                  r_O_Start:r_O_Start + 3] + '_' + comp.genome_Seq_Rev[
                                                                                                   r_O_Stop - 2:r_O_Stop + 1]
        genSeq = (comp.genome_Seq_Rev[r_G_Start:r_G_Stop + 1])
        orfSeq = (comp.genome_Seq_Rev[r_O_Start:r_O_Stop + 1])
        comp.partial_Hits.update({partial: [genSeq, orfSeq]})
    elif g_Strand == '+':
        partial = "Gene:" + str(g_Start) + '_' + str(g_Stop) + '_' + g_Strand + '_' + comp.genome_Seq[
                                                                                      g_Start - 1:g_Start + 2] + '_' + comp.genome_Seq[
                                                                                                                       g_Stop - 3:g_Stop] + ';Predicted_CDS:' + str(
            o_Start) + '_' + str(o_Stop) + '_' + g_Strand + '_' + comp.genome_Seq[
                                                                  o_Start - 1:o_Start + 2] + '_' + comp.genome_Seq[
                                                                                                   o_Stop - 3:o_Stop]
        genSeq = (comp.genome_Seq[g_Start - 1:g_Stop])
        orfSeq = (comp.genome_Seq[o_Start - 1:o_Stop])
        comp.partial_Hits.update({partial: [genSeq, orfSeq]})


def tool_comparison(ref_genes, orfs, genome, verbose):
    comp.genome_Seq = genome
    comp.genome_Seq_Rev = revCompIterative(genome)
    comp.genome_Size = len(genome)
    for gene_num, gene_details in ref_genes.items():  # Loop through each gene to compare against predicted ORFs
        g_Start = int(gene_details[0])
        g_Stop = int(gene_details[1])
        g_Strand = gene_details[2]
        g_pos = str(g_Start) + ',' + str(g_Stop)
        gene_Set = set(range(g_Start,
                             g_Stop + 1))  # Used to check Overlap of ORFs and pick best match - slow but confirms best match
        overlapping_ORFs = collections.OrderedDict()
        perfect_Match = False
        out_Frame = False
        for pos, orf_Details in orfs.items():  # Check if perfect match, if not check if match covers at least 75% of gene - Loop through ALL ORFs - SLOW
            o_Start = int(pos.split(',')[0])
            o_Stop = int(pos.split(',')[1])
            o_Strand = orf_Details[0]
            orf_Set = set(range(o_Start, o_Stop + 1))
            if o_Stop <= g_Start or o_Start >= g_Stop:  # Not caught up yet
                continue
            elif o_Start == g_Start and o_Stop == g_Stop:  # If perfect match, break and skip the rest of the ORFs
                perfect_Match = True
                break
            elif g_Start <= o_Start < g_Stop or g_Start < o_Stop < g_Stop:  # If ORF Start or Stop is between gene Start or Stop
                overlap = len(gene_Set.intersection(orf_Set))
                coverage = 100 * float(overlap) / float(len(gene_Set))
                orf_Details.append(coverage)
                if abs(o_Stop - g_Stop) % 3 == 0 and o_Strand == g_Strand and coverage >= MIN_COVERAGE:  # Only continue if ORF covers at least 75% of the gene and is in frame
                    overlapping_ORFs.update({pos: orf_Details})
                elif coverage >= MIN_COVERAGE:  # Not in frame / on same strand
                    comp.out_Of_Frame_ORFs.update({pos: orf_Details})
                    out_Frame = True
            elif o_Start <= g_Start and o_Stop >= g_Stop:  # If ORF extends one or both ends of the gene
                overlap = len(gene_Set.intersection(orf_Set))
                coverage = 100 * float(overlap) / float(len(gene_Set))
                orf_Details.append(coverage)
                if abs(o_Stop - g_Stop) % 3 == 0 and o_Strand == g_Strand and coverage >= MIN_COVERAGE:  # Only continue if ORF covers at least 75% of the gene and is in frame
                    overlapping_ORFs.update({pos: orf_Details})
                elif coverage >= MIN_COVERAGE:
                    comp.out_Of_Frame_ORFs.update({pos: orf_Details})
                    out_Frame = True
            else:
                if verbose == True:
                    print("Unexpected Error Finding Predicted CDSs")  # Should not happen
        # Now Check that we select the best ORF
        ### Multi_Match_ORFs Should contain All genes found by a specific ORF
        if perfect_Match == True:  # Check if the ORF is a perfect match to the Gene
            m_ORF_Details = orf_Details[:]
            m_ORF_Details.append(g_pos)
            if g_pos in comp.matched_ORFs.keys():
                previously_Covered_Gene = comp.matched_ORFs[g_pos][-1]
                comp.multi_Matched_ORFs[g_pos] += [g_pos.replace(',', '-'), previously_Covered_Gene.replace(',',
                                                                                                            '-')]  # ORF is same as gene so can use g_pos
            comp.matched_ORFs.update({g_pos: m_ORF_Details})
            comp.genes_Detected.update({str(gene_details): g_pos})
            match_Statistics(o_Start, o_Stop, g_Start, g_Stop, g_Strand)
            perfect_Matched_Genes(g_Start, g_Stop, g_Strand)
            if verbose == True:
                print('Perfect Match')
        elif perfect_Match == False and len(
                overlapping_ORFs) == 1:  # If we do not have a perfect match but 1 ORF which has passed the filtering
            orf_Pos = list(overlapping_ORFs.keys())[0]
            o_Start = int(orf_Pos.split(',')[0])
            o_Stop = int(orf_Pos.split(',')[1])
            orf_Details = overlapping_ORFs[orf_Pos]
            m_ORF_Details = orf_Details[:]
            m_ORF_Details.append(g_pos)
            if orf_Pos in comp.matched_ORFs.keys():
                try:
                    previously_Covered_Gene = comp.matched_ORFs[g_pos][-1]
                except KeyError:
                    last_key = [*comp.matched_ORFs.keys()][-1]
                    previously_Covered_Gene = comp.matched_ORFs[last_key][-1]
                comp.multi_Matched_ORFs[orf_Pos] += [g_pos.replace(',', '-'), previously_Covered_Gene.replace(',',
                                                                                                   '-')]  # ORF collects multiple gene pos'
            comp.matched_ORFs.update({orf_Pos: m_ORF_Details})
            comp.genes_Detected.update({str(gene_details): orf_Pos})
            match_Statistics(o_Start, o_Stop, g_Start, g_Stop, g_Strand)
            if verbose == True:
                print('Partial Match')
            partial_Hit_Calc(g_Start, g_Stop, g_Strand, o_Start, o_Stop)
        elif perfect_Match == False and len(
                overlapping_ORFs) >= 1:  # If we have more than 1 potential ORF match, we check to see which is the 'best' hit
            orf_Pos, orf_Details = candidate_ORF_Selection(gene_Set, overlapping_ORFs)  # Return best match
            o_Start = int(orf_Pos.split(',')[0])
            o_Stop = int(orf_Pos.split(',')[1])
            m_ORF_Details = orf_Details[:]
            m_ORF_Details.append(g_pos)
            if orf_Pos in comp.matched_ORFs.keys():
                try:
                    previously_Covered_Gene = comp.matched_ORFs[g_pos][-1]
                except KeyError:
                    last_key = [*comp.matched_ORFs.keys()][-1]
                    previously_Covered_Gene = comp.matched_ORFs[last_key][-1]
                comp.multi_Matched_ORFs[orf_Pos] += [g_pos.replace(',', '-'), previously_Covered_Gene.replace(',',
                                                                                                              '-')]  # ORF collects multiple gene pos'
            comp.matched_ORFs.update({orf_Pos: m_ORF_Details})
            comp.genes_Detected.update({str(gene_details): orf_Pos})
            match_Statistics(o_Start, o_Stop, g_Start, g_Stop, g_Strand)
            if verbose == True:
                print('There was more than 1 potential Match - Best Chosen')
            partial_Hit_Calc(g_Start, g_Stop, g_Strand, o_Start, o_Stop)
        elif out_Frame:  # Keep record of ORFs which overlap a gene but in the wrong frame
            if verbose == True:
                print("Out of Frame Predicted CDS")
            genes_Unmatched(g_Start, g_Stop, g_Strand)  #
        else:
            genes_Unmatched(g_Start, g_Stop, g_Strand)  # No hit
            if verbose == True:
                print("No Hit")
    for orf_Key in comp.matched_ORFs:  # Remove ORFs from out of frame if ORF was correctly matched to another Gene
        if orf_Key in comp.out_Of_Frame_ORFs:
            del comp.out_Of_Frame_ORFs[orf_Key]
    ######################################## ORF Lengths and Precision
    start_Difference = [x for x in comp.start_Difference if x != 0]  # Remove 0s (Perfect hits)
    stop_Difference = [x for x in comp.stop_Difference if x != 0]
    if len(start_Difference) >= 1:
        median_Start_Difference = np.median(start_Difference)
    else:
        median_Start_Difference = 'N/A'
    if len(stop_Difference) >= 1:
        median_Stop_Difference = np.median(stop_Difference)
    else:
        median_Stop_Difference = 'N/A'

    # Get Start and Stop Codon Usage
    atg_P, gtg_P, ttg_P, att_P, ctg_P, other_Start_P, other_Starts = start_Codon_Count(orfs)
    tag_P, taa_P, tga_P, other_Stop_P, other_Stops = stop_Codon_Count(orfs)
    # Count nucleotides found from ALL ORFs
    gene_Nuc_Array = np.zeros((comp.genome_Size), dtype=np.int)
    orf_Nuc_Array = np.zeros((comp.genome_Size), dtype=np.int)
    matched_ORF_Nuc_Array = np.zeros((comp.genome_Size), dtype=np.int)

    prev_Gene_Stop = 0
    prev_Gene_Overlapped = False
    for gene_Num, gene_details in ref_genes.items():  # Loop through each gene to compare against predicted ORFs
        g_Start = int(gene_details[0])
        g_Stop = int(gene_details[1])
        g_Strand = gene_details[2]
        gene_Length = (g_Stop - g_Start)
        comp.gene_Lengths.append(gene_Length)
        gene_Nuc_Array[g_Start - 1:g_Stop] = [1]  # Changing all between the two positions to 1's
        comp.gene_GC.append(nuc_Count(g_Start, g_Stop, g_Strand))
        if gene_Length <= SHORT_ORF_LENGTH:  # .utils
            comp.gene_Short.append(gene_Length)
        ### Calculate overlapping Genes -
        if prev_Gene_Stop > g_Start:
            if '+' in g_Strand:
                comp.gene_Pos_Olap.append(prev_Gene_Stop - g_Start)
            elif '-' in g_Strand:
                comp.gene_Neg_Olap.append(prev_Gene_Stop - g_Start)
            prev_Gene_Overlapped = True
        elif prev_Gene_Stop < g_Start:
            if prev_Gene_Overlapped == True:
                if '+' in g_Strand:
                    comp.gene_Pos_Olap.append(0)
                elif '-' in g_Strand:
                    comp.gene_Neg_Olap.append(0)
            prev_Gene_Overlapped = False
        prev_Gene_Stop = g_Stop
    if prev_Gene_Overlapped == True:  # If last has a prev overlap, count it
        if '+' in g_Strand:
            comp.gene_Pos_Olap.append(0)
        elif '-' in g_Strand:
            comp.gene_Neg_Olap.append(0)
    ####
    min_Gene_Length = min(comp.gene_Lengths)
    max_Gene_Length = max(comp.gene_Lengths)
    median_Gene_Length = np.median(comp.gene_Lengths)
    prev_ORF_Stop = 0
    prev_ORF_Overlapped = False
    for o_Positions, orf_Details in orfs.items():
        o_Start = int(o_Positions.split(',')[0])
        o_Stop = int(o_Positions.split(',')[1])
        o_Strand = orf_Details[0]
        # Stats just for Unmatched ORFs
        if o_Positions not in list(comp.matched_ORFs.keys()):
            orf_Unmatched(o_Start, o_Stop, o_Strand)
        # Get ORF Strand metrics:
        if o_Strand == "+":  # Get number of Positive and Negative strand ORFs
            comp.pos_Strand += 1
        elif o_Strand == "-":
            comp.neg_Strand += 1
        orf_Length = (o_Stop - o_Start)
        comp.orf_Lengths.append(orf_Length)
        orf_Nuc_Array[o_Start - 1:o_Stop] = [1]  # Changing all between the two positions to 1's
        comp.orf_GC.append(nuc_Count(o_Start, o_Stop, o_Strand))
        if orf_Length <= SHORT_ORF_LENGTH:  # .utils
            comp.orf_Short.append(orf_Length)
        ### Calculate overlapping ORFs -
        if prev_ORF_Stop > o_Start:
            if '+' in o_Strand:
                comp.orf_Pos_Olap.append(prev_ORF_Stop - o_Start)
            elif '-' in o_Strand:
                comp.orf_Neg_Olap.append(prev_ORF_Stop - o_Start)
            prev_ORF_Overlapped = True
        elif prev_ORF_Stop < o_Start:
            if prev_ORF_Overlapped == True:
                if '+' in o_Strand:
                    comp.orf_Pos_Olap.append(0)
                elif '-' in o_Strand:
                    comp.orf_Neg_Olap.append(0)
            prev_ORF_Overlapped = False
        prev_ORF_Stop = o_Stop
        if prev_ORF_Overlapped == True:  # If last has a prev overlap, count it
            if '+' in o_Strand:
                comp.orf_Pos_Olap.append(0)
            elif '-' in o_Strand:
                comp.orf_Neg_Olap.append(0)

    # Nucleotide Coverage calculated from ORFs matching a gene only
    matched_Prev_ORF_Stop = 0
    matched_Prev_ORF_Overlapped = False
    for mo_Positions, m_ORF_Details in comp.matched_ORFs.items():
        mo_Start = int(mo_Positions.split(',')[0])
        mo_Stop = int(mo_Positions.split(',')[1])
        mo_Strand = m_ORF_Details[0]
        mo_Length = (mo_Stop - mo_Start)
        matched_ORF_Nuc_Array[mo_Start - 1:mo_Stop] = [1]  # This is the complete matched orf not the matched orf bits

        comp.m_ORF_GC.append(nuc_Count(mo_Start, mo_Stop, mo_Strand))
        if mo_Length <= SHORT_ORF_LENGTH:  # .utils
            comp.m_ORF_Short.append(mo_Length)
        ### Calculate overlapping Matched ORFs -
        if matched_Prev_ORF_Stop > mo_Start:
            if '+' in mo_Strand:
                comp.m_ORF_Pos_Olap.append(matched_Prev_ORF_Stop - mo_Start)
            elif '-' in mo_Strand:
                comp.m_ORF_Neg_Olap.append(matched_Prev_ORF_Stop - mo_Start)
            matched_Prev_ORF_Overlapped = True
        elif matched_Prev_ORF_Stop < mo_Start:
            if matched_Prev_ORF_Overlapped == True:
                if '+' in mo_Strand:
                    comp.m_ORF_Pos_Olap.append(0)
                elif '-' in mo_Strand:
                    comp.m_ORF_Neg_Olap.append(0)
            matched_Prev_ORF_Overlapped = False
        matched_Prev_ORF_Stop = mo_Stop
    if matched_Prev_ORF_Overlapped == True:  # If last has a prev overlap, count it
        if '+' in mo_Strand:
            comp.m_ORF_Pos_Olap.append(0)
        elif '-' in mo_Strand:
            comp.m_ORF_Neg_Olap.append(0)
    ####
    gene_Coverage_Genome = format(100 * np.count_nonzero(gene_Nuc_Array) / comp.genome_Size, '.2f')
    orf_Coverage_Genome = format(100 * np.count_nonzero(orf_Nuc_Array) / comp.genome_Size, '.2f')
    matched_ORF_Coverage_Genome = format(100 * np.count_nonzero(matched_ORF_Nuc_Array) / comp.genome_Size,
                                         '.2f')  # This gets the nts which are in matched ORFs - Check below
    # matched_ORF_Nuc_AND_Gene = np.logical_and(matched_ORF_Nuc_Array,gene_Nuc_Array) + [0 for i in range(len(gene_Nuc_Array))] # This gets the nts which are in both matched ORFs and detected genes
    # matched_ORF_Coverage_Genome = format(100 * np.count_nonzero(matched_ORF_Nuc_AND_Gene) / comp.genome_Size,'.2f')

    # gene and orf nucleotide Intersection
    gene_ORF_Nuc_Intersection = np.count_nonzero(gene_Nuc_Array & orf_Nuc_Array)
    # not gene but orf nucleotides
    not_Gene_Nuc_Array = np.logical_not(gene_Nuc_Array) + [0 for i in range(
        len(gene_Nuc_Array))]  # End part to keep array as 1,0 not T,F
    not_Gene_Nuc_And_ORF_Count = np.count_nonzero(not_Gene_Nuc_Array & orf_Nuc_Array)
    # not orf nucleotides but gene
    not_ORF_Nuc_Array = np.logical_not(orf_Nuc_Array) + [0 for i in range(
        len(orf_Nuc_Array))]  # End part to keep array as 1,0 not T,F
    not_ORF_Nuc_And_Gene_Count = np.count_nonzero(not_ORF_Nuc_Array & gene_Nuc_Array)
    # not gene or orf nucleotides
    not_Gene_Nuc_Not_ORF_Nuc_Count = np.count_nonzero(not_Gene_Nuc_Array & not_ORF_Nuc_Array)
    # Nucleotide 'accuracy' - Normalised by number of nucelotides annotated by a gene
    NT_TP = format(gene_ORF_Nuc_Intersection / np.count_nonzero(gene_Nuc_Array), '.2f')
    NT_FP = format(not_Gene_Nuc_And_ORF_Count / np.count_nonzero(not_Gene_Nuc_Array), '.2f')
    NT_FN = format(not_ORF_Nuc_And_Gene_Count / np.count_nonzero(gene_Nuc_Array), '.2f')
    NT_TN = format(not_Gene_Nuc_Not_ORF_Nuc_Count / np.count_nonzero(not_Gene_Nuc_Array), '.2f')
    NT_Precision = format(gene_ORF_Nuc_Intersection / (gene_ORF_Nuc_Intersection + not_Gene_Nuc_And_ORF_Count), '.2f')
    NT_Recall = format(gene_ORF_Nuc_Intersection / (gene_ORF_Nuc_Intersection + not_ORF_Nuc_And_Gene_Count), '.2f')
    NT_False_Discovery_Rate = format(
        not_Gene_Nuc_And_ORF_Count / (not_Gene_Nuc_And_ORF_Count + gene_ORF_Nuc_Intersection), '.2f')
    ################################# Precision and Recall of whole ORFs and Genes
    TP = format(len(comp.genes_Detected) / len(ref_genes), '.2f')
    FP = format(len(comp.unmatched_ORFs) / len(ref_genes), '.2f')
    FN = format(len(comp.genes_Undetected) / len(ref_genes), '.2f')
    #################################################### Need a better way to handle 'no hits/ORFs'
    try:
        precision = format(float(TP) / (float(TP) + float(FP)), '.2f')
    except ZeroDivisionError:
        precision = format(0.00, '.2f')
    try:
        recall = format(float(TP) / (float(TP) + float(FN)), '.2f')
    except ZeroDivisionError:
        recall = format(0.00, '.2f')
    try:
        false_Discovery_Rate = format(float(FP) / (float(FP) + float(TP)), '.2f')
    except ZeroDivisionError:
        false_Discovery_Rate = 'N/A'
    min_ORF_Length = min(comp.orf_Lengths)
    max_ORF_Length = max(comp.orf_Lengths)
    median_ORF_Length = np.median(comp.orf_Lengths)

    ##########################################################################
    # Metrics - There are numerous cases where certain metric calculations may return a ZeroDivError.
    ORFs_Difference = format(100 * (len(orfs) - len(ref_genes)) / len(ref_genes), '.2f')  # Difference off +/-
    genes_Detected_Percentage = format(100 * (len(comp.genes_Detected) / len(ref_genes)), '.2f')
    matched_ORF_Percentage = format(100 * (len(comp.matched_ORFs) / len(orfs)), '.2f')
    all_ORF_Olap = (comp.orf_Pos_Olap + comp.orf_Neg_Olap)  # Combine pos and neg strand overlaps
    matched_ORF_Olap = (comp.m_ORF_Pos_Olap + comp.m_ORF_Neg_Olap)
    all_Gene_Olap = (comp.gene_Pos_Olap + comp.gene_Neg_Olap)

    if all_ORF_Olap:  # If no overlapping ORFs
        try:
            overlap_Difference = format(100 * (len(all_ORF_Olap) - len(all_Gene_Olap)) / len(all_Gene_Olap), '.2f')
            matched_Overlap_Difference = format(100 * (len(matched_ORF_Olap) - len(all_Gene_Olap)) / len(all_Gene_Olap),
                                                '.2f')
        except ZeroDivisionError:
            overlap_Difference = 'N/A'
            matched_Overlap_Difference = 'N/A'
        num_All_ORF_Olap = len(all_ORF_Olap)
        if matched_ORF_Olap:
            max_Matched_ORF_Olap = max(matched_ORF_Olap)
            matched_Median_ORF_Overlap = format(np.median(matched_ORF_Olap), '.2f')
        else:
            max_Matched_ORF_Olap = 'N/A'
            matched_Median_ORF_Overlap = 'N/A'
        max_All_ORF_Olap = max(all_ORF_Olap)
        median_ORF_Overlap = format(np.median(all_ORF_Olap), '.2f')
    else:
        overlap_Difference = 'N/A'
        matched_Overlap_Difference = 'N/A'
        num_All_ORF_Olap = 0
        max_Matched_ORF_Olap = 'N/A'
        max_All_ORF_Olap = 'N/A'
        median_ORF_Overlap = 'N/A'
        matched_Median_ORF_Overlap = 'N/A'
    if len(matched_ORF_Olap) == 0:  # -100.00 is not informative
        matched_Overlap_Difference = 'N/A'

    # Need to NA everything

    if comp.orf_Short and comp.gene_Short:  # IF Short-ORFs/Genes
        short_ORF_Difference = format(100 * (len(comp.orf_Short) - len(comp.gene_Short)) / len(comp.gene_Short), '.2f')
        matched_Short_ORF_Difference = format(
            100 * (len(comp.m_ORF_Short) - len(comp.gene_Short)) / len(comp.gene_Short), '.2f')
        num_ORF_Short = len(comp.orf_Short)
        num_Matched_ORF_Short = len(comp.m_ORF_Short)
    elif comp.orf_Short:  # If only Short-ORFs
        num_ORF_Short = len(comp.orf_Short)
        num_Matched_ORF_Short = 'N/A'
        short_ORF_Difference = (num_ORF_Short * 100)
        matched_Short_ORF_Difference = 'N/A'
    else:  # If only Short-Genes and Undetected StORFs
        comp.gene_Short
        short_ORF_Difference = 'N/A'
        matched_Short_ORF_Difference = 'N/A'
        num_ORF_Short = 0
        num_Matched_ORF_Short = 'N/A'
    if num_Matched_ORF_Short == 0:  # -100.00 is not informative
        matched_Short_ORF_Difference = 'N/A'

    median_Length_Difference = format(100 * (median_ORF_Length - median_Gene_Length) / median_Gene_Length, '.2f')
    min_Length_Difference = format(100 * (min_ORF_Length - min_Gene_Length) / min_Gene_Length, '.2f')
    max_Length_Difference = format(100 * (max_ORF_Length - max_Gene_Length) / max_Gene_Length, '.2f')
    pos_Strand_Percentage = format(comp.pos_Strand / len(orfs), '.2f')
    neg_Strand_Percentage = format(comp.neg_Strand / len(orfs), '.2f')
    median_ORF_GC = np.median(comp.orf_GC)
    matched_Median_ORF_GC = np.median(comp.m_ORF_GC)
    median_Gene_GC = np.median(comp.gene_GC)
    median_GC_Difference = format(100 * (float(median_ORF_GC) - float(median_Gene_GC)) / float(median_Gene_GC), '.2f')
    matched_Median_GC_Difference = format(
        100 * (float(matched_Median_ORF_GC) - float(median_Gene_GC)) / float(median_Gene_GC), '.2f')

    if comp.matched_ORFs:  # No ORFs detected a gene
        extended_CDS_Percentage = format(100 * comp.extended_CDS / len(comp.matched_ORFs), '.2f')
        extended_Start_Percentage = format(100 * comp.extended_Start / len(comp.matched_ORFs), '.2f')
        extended_Stop_Percentage = format(100 * comp.extended_Stop / len(comp.matched_ORFs), '.2f')
        perfect_Matches_Percentage = format(100 * len(comp.perfect_Matches) / len(comp.matched_ORFs), '.2f')
        perfect_Starts_Percentage = format(100 * comp.perfect_Starts / len(comp.matched_ORFs), '.2f')
        perfect_Stops_Percentage = format(100 * comp.perfect_Stops / len(comp.matched_ORFs), '.2f')
    else:
        # correct_Frame_Percentage = 0
        extended_CDS_Percentage = format(0.00, '.2f')
        extended_Start_Percentage = format(0.00, '.2f')
        extended_Stop_Percentage = format(0.00, '.2f')
        perfect_Matches_Percentage = format(0.00, '.2f')
        perfect_Starts_Percentage = format(0.00, '.2f')
        perfect_Stops_Percentage = format(0.00, '.2f')
    ################### Missed Genes  Metrics:
    if comp.genes_Undetected:
        mg_Starts = []
        mg_Stops = []
        mg_Lengths = []
        mg_Strands = []
        for mg, seq in comp.genes_Undetected.items():
            mg = mg.split(',')
            mg_Starts.append(mg[3])
            mg_Stops.append(mg[4])
            mg_Strands.append(mg[2])
            mg_Lengths.append(int(mg[1]) - int(mg[0]))

        mg_ATG = 100 * mg_Starts.count('ATG') / len(comp.genes_Undetected)
        mg_GTG = 100 * mg_Starts.count('GTG') / len(comp.genes_Undetected)
        mg_TTG = 100 * mg_Starts.count('TTG') / len(comp.genes_Undetected)
        mg_ATT = 100 * mg_Starts.count('ATT') / len(comp.genes_Undetected)
        mg_CTG = 100 * mg_Starts.count('CTG') / len(comp.genes_Undetected)
        mg_O_Start = 100 - (mg_ATG + mg_GTG + mg_TTG + mg_ATT + mg_CTG)
        mg_TGA = 100 * mg_Stops.count('TGA') / len(comp.genes_Undetected)
        mg_TAA = 100 * mg_Stops.count('TAA') / len(comp.genes_Undetected)
        mg_TAG = 100 * mg_Stops.count('TAG') / len(comp.genes_Undetected)
        mg_O_Stop = 100 - (mg_TGA + mg_TAA + mg_TAG)
        median_mg_Len = np.median(mg_Lengths)
        mg_Pos = mg_Strands.count('+')
        mg_Neg = mg_Strands.count('-')
        undetected_Gene_Metrics = (
        format(mg_ATG, '.2f'), format(mg_GTG, '.2f'), format(mg_TTG, '.2f'), format(mg_ATT, '.2f'),
        format(mg_CTG, '.2f'), format(mg_O_Start, '.2f'), format(mg_TGA, '.2f'), format(mg_TAA, '.2f'),
        format(mg_TAG, '.2f'), format(mg_O_Stop, '.2f'), format(median_mg_Len, '.2f'), mg_Pos, mg_Neg)
    else:
        undetected_Gene_Metrics = ''
    #################### Unmathced ORF Metrics:
    if comp.unmatched_ORFs:
        uo_Starts = []
        uo_Stops = []
        uo_Lengths = []
        uo_Strands = []
        for uo, seq in comp.unmatched_ORFs.items():
            uo = uo.split(',')
            uo_Starts.append(uo[3])
            uo_Stops.append(uo[4])
            uo_Strands.append(uo[2])
            uo_Lengths.append(int(uo[1]) - int(uo[0]))
        uo_ATG = 100 * uo_Starts.count('ATG') / len(comp.unmatched_ORFs)
        uo_GTG = 100 * uo_Starts.count('GTG') / len(comp.unmatched_ORFs)
        uo_TTG = 100 * uo_Starts.count('TTG') / len(comp.unmatched_ORFs)
        uo_ATT = 100 * uo_Starts.count('ATT') / len(comp.unmatched_ORFs)
        uo_CTG = 100 * uo_Starts.count('CTG') / len(comp.unmatched_ORFs)
        uo_O_Start = 100 - (uo_ATG + uo_GTG + uo_TTG + uo_ATT + uo_CTG)
        uo_TGA = 100 * uo_Stops.count('TGA') / len(comp.unmatched_ORFs)
        uo_TAA = 100 * uo_Stops.count('TAA') / len(comp.unmatched_ORFs)
        uo_TAG = 100 * uo_Stops.count('TAG') / len(comp.unmatched_ORFs)
        uo_O_Stop = 100 - (uo_TGA + uo_TAA + uo_TAG)
        # uo_O_Stop = 100 * uo_O_Stop  / len(comp.unmatched_ORFs) ########WHY?
        median_uo_Len = np.median(uo_Lengths)
        uo_Pos = uo_Strands.count('+')
        uo_Neg = uo_Strands.count('-')
        unmatched_ORF_Metrics = (
        format(uo_ATG, '.2f'), format(uo_GTG, '.2f'), format(uo_TTG, '.2f'), format(uo_ATT, '.2f'),
        format(uo_CTG, '.2f'), format(uo_O_Start, '.2f'), format(uo_TGA, '.2f'), format(uo_TAA, '.2f'),
        format(uo_TAG, '.2f'), format(uo_O_Stop, '.2f'), format(median_uo_Len, '.2f'), uo_Pos, uo_Neg)
    else:
        unmatched_ORF_Metrics = ''
    #################################

    all_Metrics = collections.OrderedDict(
        {'Number_of_ORFs': len(orfs), 'Percent_Difference_of_All_ORFs': ORFs_Difference,
         'Number_of_ORFs_that_Detected_a_Gene': len(comp.matched_ORFs),
         'Percentage_of_ORFs_that_Detected_a_Gene': matched_ORF_Percentage,
         'Number_of_Genes_Detected': len(comp.genes_Detected),
         'Percentage_of_Genes_Detected': genes_Detected_Percentage, 'Median_Length_of_All_ORFs': median_ORF_Length,
         'Median_Length_Difference': median_Length_Difference,
         'Minimum_Length_of_All_ORFs': min_ORF_Length, 'Minimum_Length_Difference': min_Length_Difference,
         'Maximum_Length_of_All_ORFs': max_ORF_Length, 'Maximum_Length_Difference': max_Length_Difference,
         'Median_GC_content_of_All_ORFs': format(median_ORF_GC, '.2f'),
         'Percent_Difference_of_All_ORFs_Median_GC': median_GC_Difference,
         'Median_GC_content_of_Matched_ORFs': format(matched_Median_ORF_GC, '.2f'),
         'Percent_Difference_of_Matched_ORF_GC': matched_Median_GC_Difference,
         'Number_of_ORFs_which_Overlap_Another_ORF': num_All_ORF_Olap,
         'Percent_Difference_of_Overlapping_ORFs': overlap_Difference,
         'Maximum_ORF_Overlap': max_All_ORF_Olap, 'Median_ORF_Overlap': median_ORF_Overlap,
         'Number_of_Matched_ORFs_Overlapping_Another_ORF': len(matched_ORF_Olap),
         'Percentage_Difference_of_Matched_Overlapping_CDSs': matched_Overlap_Difference,
         'Maximum_Matched_ORF_Overlap': max_Matched_ORF_Olap, 'Median_Matched_ORF_Overlap': matched_Median_ORF_Overlap,
         'Number_of_Short-ORFs': num_ORF_Short, 'Percent_Difference_of_Short-ORFs': short_ORF_Difference,
         'Number_of_Short-Matched-ORFs': num_Matched_ORF_Short,
         'Percent_Difference_of_Short-Matched-ORFs': matched_Short_ORF_Difference,
         'Number_of_Perfect_Matches': len(comp.perfect_Matches),
         'Percentage_of_Perfect_Matches': perfect_Matches_Percentage,
         'Number_of_Perfect_Starts': comp.perfect_Starts, 'Percentage_of_Perfect_Starts': perfect_Starts_Percentage,
         'Number_of_Perfect_Stops': comp.perfect_Stops, 'Percentage_of_Perfect_Stops': perfect_Stops_Percentage,
         'Number_of_Out_of_Frame_ORFs': len(comp.out_Of_Frame_ORFs),
         'Number_of_Matched_ORFs_Extending_a_Coding_Region': comp.extended_CDS,
         'Percentage_of_Matched_ORFs_Extending_a_Coding_Region': extended_CDS_Percentage,
         'Number_of_Matched_ORFs_Extending_Start_Region': comp.extended_Start,
         'Percentage_of_Matched_ORFs_Extending_Start_Region': extended_Start_Percentage,
         'Number_of_Matched_ORFs_Extending_Stop_Region': comp.extended_Stop,
         'Percentage_of_Matched_ORFs_Extending_Stop_Region': extended_Stop_Percentage,
         'Number_of_All_ORFs_on_Positive_Strand': comp.pos_Strand,
         'Percentage_of_All_ORFs_on_Positive_Strand': pos_Strand_Percentage,
         'Number_of_All_ORFs_on_Negative_Strand': comp.neg_Strand,
         'Percentage_of_All_ORFs_on_Negative_Strand': neg_Strand_Percentage,
         'Median_Start_Difference_of_Matched_ORFs': median_Start_Difference,
         'Median_Stop_Difference_of_Matched_ORFs': median_Stop_Difference, 'ATG_Start_Percentage': atg_P,
         'GTG_Start_Percentage': gtg_P, 'TTG_Start_Percentage': ttg_P,
         'ATT_Start_Percentage': att_P, 'CTG_Start_Percentage': ctg_P, 'Other_Start_Codon_Percentage': other_Start_P,
         'TAG_Stop_Percentage': tag_P, 'TAA_Stop_Percentage': taa_P,
         'TGA_Stop_Percentage': tga_P, 'Other_Stop_Codon_Percentage': other_Stop_P, 'True_Positive': TP,
         'False_Positive': FP, 'False_Negative': FN, 'Precision': precision,
         'Recall': recall, 'False_Discovery_Rate': false_Discovery_Rate, 'Nucleotide_True_Positive': NT_TP,
         'Nucleotide_False_Positive': NT_FP, 'Nucleotide_True_Negative': NT_TN,
         'Nucleotide_False_Negative': NT_FN, 'Nucleotide_Precision': NT_Precision, 'Nucleotide_Recall': NT_Recall,
         'Nucleotide_False_Discovery_Rate': NT_False_Discovery_Rate,
         'ORF_Nucleotide_Coverage_of_Genome': orf_Coverage_Genome,
         'Matched_ORF_Nucleotide_Coverage_of_Genome': matched_ORF_Coverage_Genome})

    rep_Metrics = collections.OrderedDict(
        {'Percentage_of_Genes_Detected': genes_Detected_Percentage,
         'Percentage_of_ORFs_that_Detected_a_Gene': matched_ORF_Percentage,
         'Percent_Difference_of_All_ORFs': ORFs_Difference,
         'Median_Length_Difference': median_Length_Difference,
         'Percentage_of_Perfect_Matches': perfect_Matches_Percentage,
         'Median_Start_Difference_of_Matched_ORFs': median_Start_Difference,
         'Median_Stop_Difference_of_Matched_ORFs': median_Stop_Difference,
         'Percentage_Difference_of_Matched_Overlapping_CDSs': matched_Overlap_Difference,
         'Percent_Difference_of_Short-Matched-ORFs': matched_Short_ORF_Difference,
         'Precision': precision,
         'Recall': recall,
         'False_Discovery_Rate': false_Discovery_Rate})

    # To account for unbalanced data
    for m_key, m_value in all_Metrics.items():
        if 'nan' == m_value:
            all_Metrics[m_key] = 'N/A'

    return all_Metrics, rep_Metrics, start_Difference, stop_Difference, other_Starts, other_Stops, comp.perfect_Matches, comp.genes_Undetected, comp.unmatched_ORFs, undetected_Gene_Metrics, unmatched_ORF_Metrics, orf_Coverage_Genome, matched_ORF_Coverage_Genome, gene_Coverage_Genome, comp.multi_Matched_ORFs, comp.partial_Hits



from importlib import import_module
import argparse
import collections
from datetime import date
import sys
try:
    from ORForise.src.ORForise.utils import revCompIterative
except ImportError:

     from ORForise.utils import revCompIterative



parser = argparse.ArgumentParser()
parser.add_argument('-dna', '--genome_dna', required=True, help='Genome DNA file (.fa) which both annotations '
                                                                'are based on')
parser.add_argument('-gff', '--genome_gff', required=True,
                    help='Which annotation file to add to reference annotation?')
args = parser.parse_args()




def cds_checker(genome_dna,genome_gff):
    genome_seq = ""
    with open(genome_dna, 'r') as genome_fasta:
        for line in genome_fasta:
            line = line.replace("\n", "")
            if not line.startswith('>'):
                genome_seq += str(line)
            else:
                genome_id = line.split()[0].replace('>','')

    ###########################################
    genome_size = len(genome_seq)
    genome_rev = revCompIterative(genome_seq)
    cds_dict = collections.OrderedDict()  # Order is important
    count = 0
    with open(genome_gff, 'r') as genome_gff:
        for line in genome_gff:
            line = line.split('\t')
            try:
                if "biological_region" in line[2] and len(line) == 9:
                    start = int(line[3])
                    stop = int(line[4])
                    strand = line[6]

                    if '-' in strand:  # Reverse Compliment starts and stops adjusted
                        r_start = genome_size - stop
                        r_stop = genome_size - start
                        startCodon = genome_rev[r_start:r_start + 3]
                        stopCodon = genome_rev[r_stop - 2:r_stop + 1]
                        length = abs(start - stop-1)
                    elif '+' in strand:
                        startCodon = genome_seq[start - 1:start + 2]
                        stopCodon = genome_seq[stop - 3:stop]
                        length = abs(start-1 - stop)
                    po = str(start) + ',' + str(stop)
                    orf = [strand, startCodon, stopCodon]
                    cds_dict.update({po: orf})

                    if length % 3 == 0:
                        print("In-Fame")
                    else:
                        sys.exit("W")


                elif "bio" in line[2]:
                    sys.exit("SAS")
            except IndexError:
                continue


if __name__ == "__main__":
    cds_checker(**vars(args))

    print("Complete")



import collections

try:
    from utils import revCompIterative
    from utils import sortORFs
except ImportError:
    from ORForise.utils import revCompIterative
    from ORForise.utils import sortORFs


def MetaGeneAnnotator(tool_pred, genome):
    metaGeneAnnotator_ORFs = collections.OrderedDict()
    genome_size = len(genome)
    genome_rev = revCompIterative(genome)
    with open(tool_pred, 'r') as MetaGeneAnnotator_input:
        for line in MetaGeneAnnotator_input:
            line = line.split()
            if len(line) == 11:
                if "gene_" in line[0]:
                    start = int(line[1])
                    stop = int(line[2])
                    strand = line[3]
                    if '-' in strand:  # Reverse Compliment starts and stops adjusted
                        r_start = genome_size - stop
                        r_stop = genome_size - start
                        startCodon = genome_rev[r_start:r_start + 3]
                        stopCodon = genome_rev[r_stop - 2:r_stop + 1]
                    elif '+' in strand:
                        startCodon = genome[start - 1:start + 2]
                        stopCodon = genome[stop - 3:stop]
                    po = str(start) + ',' + str(stop)
                    orf = [strand, startCodon, stopCodon, 'CDS']
                    metaGeneAnnotator_ORFs.update({po: orf})

    metaGeneAnnotator_ORFs = sortORFs(metaGeneAnnotator_ORFs)
    return metaGeneAnnotator_ORFs



import argparse
import collections

parser = argparse.ArgumentParser()
parser.add_argument('-t', '--tool', required=True, help='Which tool to compare?')
parser.add_argument('-p', '--parameters', required=False, help='Optional parameters for prediction tool.')
parser.add_argument('-g', '--genome', required=True, help='Which genome to analyse?')
args = parser.parse_args()


def revCompIterative(watson):
    complements = {'A': 'T', 'T': 'A', 'C': 'G', 'G': 'C', 'N': 'N'}
    watson = watson.upper()
    watsonrev = watson[::-1]
    crick = ""
    for nt in watsonrev:
        crick += complements[nt]
    return crick


def partial_gene_read(results_file, partial_genes):
    # partial Genes Read-In
    orf_Lengths = []
    gene_Lengths = []
    read = False
    prev = ''
    for line in results_file:
        line = line.strip()
        if read == True:
            if line.startswith('Gene:'):
                line = line.replace('Gene:', '')
                entry = line.split('_')
                g_Pos = entry[0] + '_' + entry[1]
                strand = entry[2]
                prev = 'Gene'
            elif line.startswith('ORF:'):
                line = line.replace('ORF:', '')
                entry = line.split('_')
                o_Pos = entry[0] + '_' + entry[1]
                prev = 'ORF'
            elif line:
                if 'Gene' in prev:
                    g_Seq = line.strip()
                    gene_Lengths.append(len(g_Seq))
                elif 'ORF' in prev:
                    o_Seq = line.strip()
                    orf_Lengths.append(len(o_Seq))
            elif not line:
                partial_genes.update({g_Pos: [strand, g_Seq, o_Pos, o_Seq]})
        if line.startswith('Partial_Gene_Hits:'):
            read = True
    return partial_genes, orf_Lengths, gene_Lengths


def perfect_Matches(results_file, perfect_Match_Genes):
    read = False
    for line in results_file:
        line = line.strip()
        if read == True:
            if line.startswith('>'):
                entry = line.split('_')
                g_Pos = entry[1] + '_' + entry[2]
                strand = entry[3]
            elif line:
                g_Seq = line
                g_Start = line[0:3]
                g_Stop = line[-3:]
            elif not line:
                perfect_Match_Genes.update({g_Pos: [strand, g_Seq, g_Start, g_Stop]})
        if line.startswith('Perfect_Match_Genes:'):
            read = True
        if line.startswith('Undetected_Genes'):
            break
    return perfect_Match_Genes


def detail_transfer(genes, partial_genes):
    for partial, m_details in partial_genes.items():
        try:
            details = genes[partial]
            gc = details[2]
            up_Overlap = details[3]
            down_Overlap = details[4]
            m_details.insert(2, gc)
            m_details.insert(3, up_Overlap)
            m_details.insert(4, down_Overlap)
        except KeyError:
            pass
    return partial_genes


def result_compare(results_file, genome_file):
    genome = ""
    with open('../Genomes/' + genome_file + '.fa', 'r') as genome_file:
        for line in genome_file:
            line = line.replace("\n", "")
            if ">" not in line:
                genome += str(line)

    partial_genes = collections.OrderedDict()
    perfect_Match_Genes = collections.OrderedDict()
    partial_genes, orf_Lengths, gene_Lengths = partial_gene_read(results_file, partial_genes)
    results_file.seek(0)
    perfect_Match_Genes = perfect_Matches(results_file, perfect_Match_Genes)

    perfect_Match_Gene_Start_Codons = collections.OrderedDict({'ATG': 0, 'GTG': 0, 'TTG': 0, 'CTG': 0, 'Other': 0})
    for gene, data in perfect_Match_Genes.items():
        try:
            perfect_Match_Gene_Start_Codons[data[2]] += 1
        except  KeyError:
            perfect_Match_Gene_Start_Codons['Other'] += 1
    print("Perfect Match Start Codons\nATG:" + str(perfect_Match_Gene_Start_Codons['ATG']) + ",GTG:" + str(
        perfect_Match_Gene_Start_Codons['GTG']) + ",TTG:" +
          str(perfect_Match_Gene_Start_Codons['TTG']) + ",CTG:" + str(
        perfect_Match_Gene_Start_Codons['CTG']) + ",Other_Start:" + str(perfect_Match_Gene_Start_Codons['Other']))

    strands = collections.defaultdict(int, {'-': 0, '+': 0})
    start_Codon_Substitution = collections.OrderedDict(
        {'ATG-ATG': 0, 'GTG-ATG': 0, 'TTG-ATG': 0, 'CTG-ATG': 0, 'Alt-ATG': 0,
         'ATG-GTG': 0, 'GTG-GTG': 0, 'TTG-GTG': 0, 'CTG-GTG': 0, 'Alt-CTG': 0,
         'ATG-TTG': 0, 'GTG-TTG': 0, 'TTG-TTG': 0, 'CTG-TTG': 0, 'Alt-GTG': 0,
         'ATG-CTG': 0, 'GTG-CTG': 0, 'TTG-CTG': 0, 'CTG-CTG': 0, 'Alt-TTG': 0,
         'ATG-Alt': 0, 'GTG-Alt': 0, 'TTG-Alt': 0, 'CTG-Alt': 0, 'Alt-Alt': 0})

    codon_set = ['ATG', 'CTG', 'GTG', 'TTG']
    for gene, data in partial_genes.items():
        strands[data[0]] += 1
        gene_Start = [data[1][0:3]]
        orf_Start = [data[3][0:3]]
        if gene_Start[0] in codon_set:
            gene_Start = gene_Start[0]
        else:
            print('Gene_Codon_Alternative:' + str(gene_Start[0]))
            gene_Start = 'Alt'
        if orf_Start[0] in codon_set:
            orf_Start = orf_Start[0]
        else:
            print('ORF_Codon_Alternative:' + str(orf_Start[0]))
            orf_Start = 'Alt'

        matrix_index = gene_Start + '-' + orf_Start
        start_Codon_Substitution[matrix_index] += 1

    ####### HERE - Need to flip the data  - GS along the top
    subs = start_Codon_Substitution.values()
    subs = list(subs)
    subs[:0] = ['ATG', 'GTG', 'TTG', 'CTG', 'Other']
    for i in [subs[c:c + 5] for c in range(0, len(subs), 5) if c % 5 == 0]:
        print(*i)


if __name__ == "__main__":
    options = parser.parse_args()
    parameters = options.parameters
    tool = options.tool
    genome = options.genome
    if parameters:
        results_file = open('../Tools/' + tool + '/' + tool + '_' + genome + '_' + parameters + '.csv')
    else:
        results_file = open('../Tools/' + tool + '/' + tool + '_' + genome + '.csv')
    result_compare(results_file, genome)



from importlib import import_module
import argparse
import collections
import csv,sys
#####
try:
    from Comparator import tool_comparison
except ImportError:
    from .Comparator import tool_comparison

try:
    from utils import *
except ImportError:
    from ORForise.utils import *

##########################

def comparator(options):
    genome_Seq = ""
    with open(options.genome_DNA, 'r') as genome:
        for line in genome:
            line = line.replace("\n", "")
            if not line.startswith('>'):
                genome_Seq += str(line)
    ##############################################
    if not options.reference_tool:  # IF using Ensembl for comparison
        ref_genes = collections.OrderedDict()  # Order is important
        count = 0
        with open(options.reference_annotation, 'r') as genome_gff:
            for line in genome_gff:
                line = line.split('\t')
                try:
                    if "CDS" in line[2] and len(line) == 9:
                        start = int(line[3])
                        stop = int(line[4])
                        strand = line[6]
                        gene_details = [start,stop,strand]
                        ref_genes.update({count:gene_details})
                        count += 1
                except IndexError:
                    continue
        ref_genes = sortGenes(ref_genes) # sorted GFF refernce
    else:  # IF using a tool as reference
        try:
            reference_tool_ = import_module('Tools.' + options.reference_tool + '.' + options.reference_tool,
                                             package='my_current_pkg')
        except ModuleNotFoundError:
            try:
                reference_tool_ = import_module('ORForise.Tools.' + options.reference_tool + '.' + options.reference_tool,
                                             package='my_current_pkg')
            except ModuleNotFoundError:
                sys.exit("Tool not available")
        reference_tool_ = getattr(reference_tool_, options.reference_tool)
        ############ Reformatting tool output for ref_genes
        ref_genes_tmp = reference_tool_(options.reference_annotation, genome_Seq)
        ref_genes = collections.OrderedDict()
        for i, (pos, details) in enumerate(ref_genes_tmp.items()):
            pos = pos.split(',')
            ref_genes.update({i:[pos[0],pos[1],details[0]]})
    #############################################
    try:
        tool_ = import_module('Tools.' + options.tool + '.' + options.tool, package='my_current_pkg')
    except ModuleNotFoundError:
        try:
            tool_ = import_module('ORForise.Tools.' + options.tool + '.' + options.tool, package='my_current_pkg')
        except ModuleNotFoundError:
            sys.exit("Tool not available - Did you get the name right?")
    tool_ = getattr(tool_, options.tool)
    orfs = tool_(options.tool_prediction, genome_Seq)
    all_Metrics, all_rep_Metrics, start_precision, stop_precision, other_starts, other_stops, perfect_Matches, missed_genes, unmatched_orfs, undetected_gene_metrics, unmatched_orf_metrics, orf_Coverage_Genome, matched_ORF_Coverage_Genome, gene_coverage_genome, multi_Matched_ORFs, partial_Hits = tool_comparison(
        ref_genes, orfs, genome_Seq, options.verbose)
    ############################################# To get default output filename from input file details
    genome_name = options.reference_annotation.split('/')[-1].split('.')[0]
    metric_description = list(all_Metrics.keys())
    metrics = list(all_Metrics.values())
    rep_metric_description = list(all_rep_Metrics.keys())
    rep_metrics = list(all_rep_Metrics.values())
    ############## Printing to std-out and optional csv file
    print('Genome Used: ' + str(options.reference_annotation.split('/')[-1]))
    if options.reference_tool:
        print('Reference Tool Used: '+str(options.reference_tool))
    else:
        print('Reference Used: ' + str(options.reference_annotation))
    print('Tool Compared: '+str(options.tool))
    print('Perfect Matches: ' + str(len(perfect_Matches)) + ' [' + str(len(ref_genes))+ '] - '+ format(100 * len(perfect_Matches)/len(ref_genes),'.2f')+'%')
    print('Partial Matches: ' + str(len(partial_Hits)) + ' [' + str(len(ref_genes))+ '] - '+ format(100 * len(partial_Hits)/len(ref_genes),'.2f')+'%')
    print('Missed Genes: ' + str(len(missed_genes)) + ' [' + str(len(ref_genes))+ '] - '+ format(100 * len(missed_genes)/len(ref_genes),'.2f')+'%')
    if options.outname:
        with open(options.outname, 'w', newline='\n', encoding='utf-8') as out_file:  # Clear write out of report
            tool_out = csv.writer(out_file, quoting=csv.QUOTE_NONE, escapechar=" ")
            tool_out.writerow(['Representative_Metrics:'])
            tool_out.writerow(rep_metric_description)
            tool_out.writerow(rep_metrics)
            tool_out.writerow(['All_Metrics:'])
            tool_out.writerow(metric_description)
            tool_out.writerow(metrics)
            tool_out.writerow(['Reference_CDS_Gene_Coverage_of_Genome'])
            tool_out.writerow([gene_coverage_genome])
            tool_out.writerow(['Predicted_CDS_Coverage_of_Genome'])
            tool_out.writerow([orf_Coverage_Genome])
            tool_out.writerow(['Matched_Predicted_CDS_Coverage_of_Genome'])
            tool_out.writerow([matched_ORF_Coverage_Genome])
            tool_out.writerow(['Start_Position_Difference:'])
            tool_out.writerow(start_precision)
            tool_out.writerow(['Stop_Position_Difference:'])
            tool_out.writerow(stop_precision)
            tool_out.writerow(['Alternative_Starts_Predicted:'])
            tool_out.writerow(other_starts)
            tool_out.writerow(['Alternative_Stops_Predicted:'])
            tool_out.writerow(other_stops)
            tool_out.writerow(['Undetected_Gene_Metrics:'])
            tool_out.writerow([
                                  'ATG_Start,GTG_Start,TTG_Start,ATT_Start,CTG_Start,Alternative_Start_Codon,TGA_Stop,TAA_Stop,TAG_Stop,Alternative_Stop_Codon,Median_Length,ORFs_on_Positive_Strand,ORFs_on_Negative_Strand'])
            tool_out.writerow(undetected_gene_metrics)
            ####
            tool_out.writerow(['Perfect_Match_Genes:'])
            for key, value in perfect_Matches.items():
                key = key.split(',')
                id = ('>' + genome_name + '_' + key[0] + '_' + key[1] + '_' + key[2])
                tool_out.writerow([id + '\n' + value + '\n'])
            ####
            tool_out.writerow(['Partial_Match_Genes:'])
            for key, seqs in partial_Hits.items():
                key = key.split(';')
                gene_Seq = seqs[0]
                orf_Seq = seqs[1]
                partial = (key[0] + '\n' + gene_Seq + '\n' + key[1] + '\n' + orf_Seq + '\n')
                tool_out.writerow([partial])
            ####
            tool_out.writerow(['\nMissed_Genes:'])
            for key, value in missed_genes.items():
                key = key.split(',')
                id = ('>' + genome_name + '_' + key[0] + '_' + key[1] + '_' + key[2])
                tool_out.writerow([id + '\n' + value + '\n'])
            tool_out.writerow(['\nPredicted_CDSs_Without_Corresponding_Gene_In_Reference_Metrics:'])
            tool_out.writerow([
                                  'ATG_Start,GTG_Start,TTG_Start,ATT_Start,CTG_Start,Alternative_Start_Codon,TGA_Stop,TAA_Stop,TAG_Stop,Alternative_Stop_Codon,Median_Length,ORFs_on_Positive_Strand,ORFs_on_Negative_Strand'])
            tool_out.writerow(unmatched_orf_metrics)
            tool_out.writerow(['Predicted_CDS_Without_Corresponding_Gene_in_Reference:'])
            for key, value in unmatched_orfs.items():
                key = key.split(',')
                id = ('>' + options.tool + '_' + key[0] + '_' + key[1] + '_' + key[2])
                tool_out.writerow([id + '\n' + value])
            tool_out.writerow(['\nPredicted_CDSs_Which_Detected_more_than_one_Gene:'])

            try:
                for key, value in multi_Matched_ORFs.items():
                    key = key.split(',')
                    multi = ('Predicted_CDS:' + key[0] + '-' + key[1] + '_Genes:' + '|'.join(value))
                    tool_out.writerow([multi])
            except IndexError:
                pass

def main():
    print("Thank you for using ORForise\nPlease report any issues to: https://github.com/NickJD/ORForise/issues\n#####")

    parser = argparse.ArgumentParser(description='ORForise ' + ORForise_Version + ': Annotatione-Compare Run Parameters.')
    parser._action_groups.pop()

    required = parser.add_argument_group('Required Arguments')
    required.add_argument('-dna', dest='genome_DNA', required=True, help='Genome DNA file (.fa) which both annotations '
                                                                    'are based on')
    required.add_argument('-ref', dest='reference_annotation', required=True,
                        help='Which reference annotation file to use as reference?')
    required.add_argument('-t', dest='tool', required=True, help='Which tool to analyse? (Prodigal)')
    required.add_argument('-tp', dest='tool_prediction', required=True,
                        help='Tool genome prediction file (.gff) - Different Tool Parameters'
                             ' are compared individually via separate files')

    optional = parser.add_argument_group('Optional Arguments')
    optional.add_argument('-rt', dest='reference_tool', required=False,
                          help='What type of Annotation to compare to? -- Leave blank for Ensembl reference'
                               '- Provide tool name to compare output from two tools')

    output = parser.add_argument_group('Output')
    output.add_argument('-o', dest='outname', required=False,
                        help='Define full output filename (format is CSV) - If not provided, summary will be printed to std-out')

    misc = parser.add_argument_group('Misc')
    misc.add_argument('-v', dest='verbose', default='False', type=eval, choices=[True, False],
                      help='Default - False: Print out runtime status')
    options = parser.parse_args()

    comparator(options)

if __name__ == "__main__":
    main()
    print("Complete")



from importlib import import_module
import argparse
import collections
from datetime import date
import sys
try:
    from utils import *
except ImportError:
    from .utils import *


########################################


def gff_writer(genome_ID, genome_DNA, reference_annotation, reference_tool, ref_gene_set, additional_annotation, additional_tool, combined_ORFs, output_file):
    write_out = open(output_file, 'w')
    write_out.write('##sequence-region ' + genome_ID + ' 1 ' + str(len(genome_DNA)) + '\n')
    write_out.write("##gff-version\t3\n#\tGFF-Adder\n#\tRun Date:" + str(date.today()) + '\n')
    write_out.write("##Genome DNA File:" + genome_DNA + '\n')
    write_out.write("##Original File: " + reference_annotation + "\n##Additional File: " + additional_annotation + '\n')
    storf_num = 0
    for pos, data in combined_ORFs.items():
        pos_ = pos.split(',')
        start = pos_[0]
        stop = pos_[-1]
        strand = data[0]
        length = int(stop) - int(start)
        if pos not in ref_gene_set:  # Check if ref or additional
            type = additional_tool
            entry = (
                        genome_ID + '\t' + type + '\tCDS\t' + start + '\t' + stop + '\t.\t' + strand + '\t.\tID=Additional_Annotation_StORF_' + str(storf_num) + ';Length=' + str(length) +  '\n')
            storf_num +=1
        else:
            type = reference_tool
            entry = (
                        genome_ID + '\t' + type + '\t' + data[2] + '\t' + start + '\t' + stop + '\t.\t' + strand + '\t.\tID=Original_Annotation_' + data[1] + ';Length=' + str(length) +  '\n')
        write_out.write(entry)


def gff_adder(options):#genome_DNA, reference_tool, reference_annotation, additional_tool, additional_annotation, gene_ident, overlap, output_file):  # Only works for single contig genome
    genome_seq = ""
    with open(options.genome_DNA, 'r') as genome_fasta:
        for line in genome_fasta:
            line = line.replace("\n", "")
            if not line.startswith('>'):
                genome_seq += str(line)
            else:
                genome_ID = line.split()[0].replace('>','')
    ###########################################
    if not options.reference_tool:  # IF using Ensembl for comparison
        ref_genes = collections.OrderedDict()  # Order is important
        count = 0
        with open(options.reference_annotation, 'r') as genome_gff:
            for line in genome_gff:
                line = line.split('\t')
                try:
                    if 'CDS' in options.gene_ident and len(options.gene_ident) == 1:
                        if "CDS" in line[2] and len(line) == 9:
                            start = int(line[3])
                            stop = int(line[4])
                            strand = line[6]
                            pos = str(start)+','+str(stop)
                            ref_genes.update({pos:[strand,'ref','CDS']})
                            count += 1
                    else:
                        gene_types = options.gene_ident.split(',')
                        if any(gene_type in line[2] for gene_type in gene_types):  # line[2] for normalrun
                            start = int(line[3])
                            stop = int(line[4])
                            strand = line[6]
                            pos = str(start) + ',' + str(stop)
                            ref_genes.update({pos: [strand, 'ref',line[2]]}) #Report what type of gene/rRNA etc we have here
                            count += 1
                except IndexError:
                    continue
    elif options.reference_tool: # IF using a tool as reference
        if 'StORF_Reporter' == options.reference_tool:
            reference_tool = 'StORF_Reporter'
        try:
            reference_tool_ = import_module('Tools.' + reference_tool + '.' + reference_tool,
                                             package='my_current_pkg')
        except ModuleNotFoundError:
            try:
                reference_tool_ = import_module('ORForise.Tools.' + reference_tool + '.' + reference_tool,
                                             package='my_current_pkg')
            except ModuleNotFoundError:
                sys.exit("Tool not available")
        reference_tool_ = getattr(reference_tool_, reference_tool)
        ############ Reformatting tool output for ref_genes
        ref_genes = reference_tool_(reference_annotation=options.reference_annotation, genome_seq=genome_seq,gene_ident=options.options.gene_ident)
    ref_gene_set = list(ref_genes.keys())
    ################ Get Additional Tool'
    # if 'StORF_Reporter' == options.additional_tool:
    #     additional_tool = 'StORF_Reporter'
    try:
        additional_tool_ = import_module('Tools.' + options.additional_tool + '.' + options.additional_tool,
                                        package='my_current_pkg')
    except ModuleNotFoundError:
        try:
            additional_tool_ = import_module('ORForise.Tools.' + options.additional_tool + '.' + options.additional_tool,
                                            package='my_current_pkg')
        except ModuleNotFoundError:
            sys.exit("Tool not available")
    additional_tool_ = getattr(additional_tool_, options.additional_tool)
    additional_orfs = additional_tool_(options.additional_annotation, genome_seq)#,gene_ident=options.gene_ident)
    orfs_to_remove = []
    for orf in additional_orfs.keys():
        o_start = int(orf.split(',')[0])
        o_stop = int(orf.split(',')[1])
        orf_set = set(range(int(o_start), int(o_stop) + 1))
        for pos, details in ref_genes.items():  # Loop through each gene to compare against predicted ORFs
            g_start = int(pos.split(',')[0])
            g_stop = int(pos.split(',')[1])
            gene_set = set(range(int(g_start), int(g_stop) + 1))
            cov = len(orf_set.intersection(gene_set))
            if cov >= options.overlap:
                orfs_to_remove.append(str(o_start) + ',' + str(o_stop))
            if g_start > o_stop:
                break
    for orf_key in orfs_to_remove:  # Remove ORFs from out of frame if ORF was correctly matched to another Gene
        if orf_key in additional_orfs:
            del additional_orfs[orf_key]
    #########################################################
    combined_ORFs = {**ref_genes, **additional_orfs}
    combined_ORFs = sortORFs(combined_ORFs)

    if not options.reference_tool:
        options.reference_tool = 'Reference_Annotation'
    gff_writer(genome_ID, options.genome_DNA, options.reference_annotation, options.reference_tool, ref_gene_set, options.additional_annotation, options.additional_tool, combined_ORFs, options.output_file)


def main():
    print("Thank you for using ORForise\nPlease report any issues to: https://github.com/NickJD/ORForise/issues\n#####")

    parser = argparse.ArgumentParser(description='ORForise ' + ORForise_Version + ': GFF-Adder Run Parameters.')
    parser._action_groups.pop()

    required = parser.add_argument_group('Required Arguments')
    required.add_argument('-dna', dest='genome_DNA', required=True, help='Genome DNA file (.fa) which both annotations '
                                                                    'are based on')
    required.add_argument('-ref', dest='reference_annotation', required=True,
                        help='Which reference annotation file to use as reference?')
    required.add_argument('-at', dest='additional_tool', required=True,
                        help='Which format to use for additional annotation?')
    required.add_argument('-add', dest='additional_annotation', required=True,
                        help='Which annotation file to add to reference annotation?')
    required.add_argument('-o', dest='output_file', required=True,
                        help='Output filename')

    optional = parser.add_argument_group('Optional Arguments')
    optional.add_argument('-rt', dest='reference_tool', required=False,
                        help='Which tool format to use as reference? - If not provided, will default to '
                             'standard Ensembl GFF format, can be Prodigal or any of the other tools available')
    optional.add_argument('-gi', dest='gene_ident', default='CDS', required=False,
                        help='Identifier used for extraction of "genic" regions from reference annotation '
                             '"CDS,rRNA,tRNA": Default for is "CDS"')
    optional.add_argument('-gene_ident', action='store', dest='gene_ident', default='CDS',
                        help='Identifier used for identifying genomic features in reference annotation "CDS,rRNA,tRNA"')
    optional.add_argument('-olap', dest='overlap', default=50, type=int, required=False,
                        help='Maximum overlap between reference and additional genic regions (CDS,rRNA etc) - Default: 50 nt')

    options = parser.parse_args()

    gff_adder(options)


if __name__ == "__main__":
    main()
    print("Complete")



import argparse
import collections
import numpy as np

from ORForise.src.ORForise.utils import *

parser = argparse.ArgumentParser()
parser.add_argument('-t', '--tool', required=True, help='Which tool to compare?')
parser.add_argument('-p', '--parameters', required=False, help='Optional parameters for prediction tool.')
parser.add_argument('-g', '--genome', required=True, help='Which genome to analyse?')

args = parser.parse_args()


def gc_count(dna):
    c = 0
    a = 0
    g = 0
    t = 0
    n = 0
    for i in dna:
        if "C" in i:
            c += 1
        elif "G" in i:
            g += 1
        elif "A" in i:
            a += 1
        elif "T" in i:
            t += 1
        elif "N" in i:
            n += 1
    gc_content = format((g + c) * 100 / (a + t + g + c + n), '.2f')
    n_per = n * 100 / (a + t + g + c + n)
    return n_per, gc_content


def revCompIterative(watson):
    complements = {'A': 'T', 'T': 'A', 'C': 'G', 'G': 'C', 'N': 'N'}
    watson = watson.upper()
    watsonrev = watson[::-1]
    crick = ""
    for nt in watsonrev:
        crick += complements[nt]
    return crick


def start_Codon_Count(orfs):
    atg, gtg, ttg, att, ctg, other = 0, 0, 0, 0, 0, 0
    other_Starts = []
    for orf in orfs.values():
        codon = orf[-2]
        if codon == 'ATG':
            atg += 1
        elif codon == 'GTG':
            gtg += 1
        elif codon == 'TTG':
            ttg += 1
        elif codon == 'ATT':
            att += 1
        elif codon == 'CTG':
            ctg += 1
        else:
            other += 1
            other_Starts.append(codon)
    atg_P = format(100 * atg / len(orfs), '.2f')
    gtg_P = format(100 * gtg / len(orfs), '.2f')
    ttg_P = format(100 * ttg / len(orfs), '.2f')
    att_P = format(100 * att / len(orfs), '.2f')
    ctg_P = format(100 * ctg / len(orfs), '.2f')
    other_Start_P = format(100 * other / len(orfs), '.2f')
    return atg_P, gtg_P, ttg_P, att_P, ctg_P, other_Start_P, other_Starts


def stop_Codon_Count(orfs):
    tag, taa, tga, other = 0, 0, 0, 0
    other_Stops = []
    for orf in orfs.values():
        codon = orf[-1]
        if codon == 'TAG':
            tag += 1
        elif codon == 'TAA':
            taa += 1
        elif codon == 'TGA':
            tga += 1
        else:
            other += 1
            other_Stops.append(codon)
    tag_p = format(100 * tag / len(orfs), '.2f')
    taa_p = format(100 * taa / len(orfs), '.2f')
    tga_p = format(100 * tga / len(orfs), '.2f')
    other_Stop_P = format(100 * other / len(orfs), '.2f')
    return tag_p, taa_p, tga_p, other_Stop_P, other_Stops


def missed_gene_read(results_file, missed_genes):
    # Missed Genes Read-In
    read = False
    for line in results_file:
        if line.startswith('ORFs_Without_Corresponding_Gene_In_Ensembl_Metrics:'):
            break
        line = line.strip()
        if read == True:
            if line.startswith('>'):
                entry = line.split('_')
                entry = entry[1] + '_' + entry[2]
            elif len(line.strip()) > 0:
                startCodon = line[0:3]
                stopCodon = line[-3:]
                length = len(line)
                missed_genes.update({entry: [line, length, startCodon, stopCodon]})
        if line.startswith('Undetected_Genes:'):
            read = True

    return missed_genes


def detail_transfer(genes, missed_genes):
    for missed, m_details in missed_genes.items():
        try:
            details = genes[missed]
            gc = details[2]
            up_Overlap = details[3]
            down_Overlap = details[4]
            m_details.insert(2, gc)
            m_details.insert(3, up_Overlap)
            m_details.insert(4, down_Overlap)
        except KeyError:
            pass
    return missed_genes


def result_compare(genome, results_file):
    genome_Seq = ""
    with open('../Genomes/' + genome + '.fa', 'r') as genome_file:
        for line in genome_file:
            line = line.replace("\n", "")
            if ">" not in line:
                genome_Seq += str(line)

    missed_genes = collections.OrderedDict()
    missed_genes = missed_gene_read(results_file, missed_genes)
    list_MG = list(missed_genes.keys())
    # Analysis
    genome_Rev = revCompIterative(genome_Seq)
    genome_Size = len(genome_Seq)
    genes = collections.OrderedDict()
    count = 0
    prev_Stop = 0
    ### Record Missed and Detected Gene metrics
    genes_strand = collections.defaultdict(int)
    genes_Missed_strand = collections.defaultdict(int)
    short_PCGs, short_Missed_PCGs, pcg_GC, pcg_Missed_GC, lengths_PCG, lengths_Missed_PCG, genes_Overlap, genes_Missed_Overlap = [], [], [], [], [], [], [], []
    with open('../Genomes/' + genome + '.gff', 'r') as genome_gff:
        for line in genome_gff:
            line = line.split('\t')
            try:
                if "CDS" in line[2] and len(line) == 9:
                    start = int(line[3])
                    stop = int(line[4])
                    strand = line[6]
                    gene = str(start) + ',' + str(stop) + ',' + strand
                    if strand == '-':
                        r_Start = genome_Size - stop
                        r_Stop = genome_Size - start
                        seq = (genome_Rev[r_Start:r_Stop + 1])
                    elif strand == '+':
                        seq = (genome_Seq[start - 1:stop])
                    startCodon = seq[0:3]
                    stopCodon = seq[-3:]
                    length = stop - start
                    n_per, gc = gc_count(seq)
                    pos = str(start) + '_' + str(stop)
                    if pos in list_MG:
                        genes_Missed_strand[strand] += 1
                        pcg_Missed_GC.append(float(gc))
                        lengths_Missed_PCG.append(length)
                        if length < SHORT_ORF_LENGTH:
                            short_Missed_PCGs.append(gene)
                        if prev_Stop > start:
                            overlap = prev_Stop - start
                            genes_Missed_Overlap.append(overlap)
                        else:
                            overlap = 0
                    elif pos not in list_MG:
                        genes_strand[strand] += 1
                        pcg_GC.append(float(gc))
                        lengths_PCG.append(length)
                        if length < SHORT_ORF_LENGTH:
                            short_PCGs.append(gene)
                        if prev_Stop > start:
                            overlap = prev_Stop - start
                            genes_Overlap.append(overlap)
                        else:
                            overlap = 0

                    count += 1
                    prev_Stop = stop
                    pos = str(start) + '_' + str(stop)
                    if genes:
                        prev_details = genes[prev_pos]
                        prev_details.insert(4, overlap)
                        genes.update({prev_pos: prev_details})

                    genes.update({pos: [strand, length, gc, overlap, seq, startCodon, stopCodon]})
                    prev_pos = pos

            except IndexError:
                continue

    missed_genes = detail_transfer(genes, missed_genes)

    missed_lengths = []
    gene_lengths = []
    for key in list_MG:
        ### printed out to confirm figure lengths
        start = key.split('_')[0]
        stop = key.split('_')[1]
        m_length = int(stop) - int(start)
        missed_lengths.append(m_length)
        if key in genes:
            del genes[key]

    ## Printed out for Figure of gene lengths
    for key in genes.keys():
        start = key.split('_')[0]
        stop = key.split('_')[1]
        g_length = int(stop) - int(start)
        gene_lengths.append(g_length)

    print("Number of Genes Missed:" + str(len(missed_lengths)) + '\nLengths of Genes Missed:\n' + str(missed_lengths))

    median_PCG = np.median(lengths_PCG)
    median_PCG_Olap = np.median(genes_Overlap)
    longest_Olap = max(genes_Overlap)
    num_overlaps = len(genes_Overlap)
    gc_median = format(np.median(pcg_GC), '.2f')
    num_Short_PCGs = len(short_PCGs)

    atg_P, gtg_P, ttg_P, att_P, ctg_P, other_Starts_P, other_Starts = start_Codon_Count(genes)
    tag_P, taa_P, tga_P, other_Stops_P, other_Stops = stop_Codon_Count(genes)
    m_atg_P, m_gtg_P, m_ttg_P, m_att_P, m_ctg_P, m_other_Starts_P, m_other_Starts = start_Codon_Count(missed_genes)
    m_tag_P, m_taa_P, m_tga_P, m_other_Stops_P, m_other_Stops = stop_Codon_Count(missed_genes)

    output = ("Number of Missed Protein Coding Genes in " + str(genome) + " : " + str(
        len(lengths_PCG)) + ", Median Length of PCGs: " +
              str(median_PCG) + ", Min Length of PCGs: " + str(min(lengths_PCG)) + ", Max Length of PCGs: " + str(
                max(lengths_PCG)) +
              ", Number of PCGs on Pos Strand: " + str(
                genes_Missed_strand['+']) + ", Number of PCGs on Neg Strand: " + str(genes_Missed_strand['-']) +
              ", Median GC of PCGs: " + str(gc_median) + ", Number of Overlapping PCGs: " + str(num_overlaps) +
              ", Longest PCG Overlap: " + str(longest_Olap) + ", Median PCG Overlap: " + str(median_PCG_Olap) +
              ", Number of PCGs less than 100nt: " + str(num_Short_PCGs) +

              '\nPercentage of Genes starting with ATG - Annotation/Missed: ' + atg_P + ' ' + m_atg_P +
              '\nPercentage of Genes starting with GTG - Annotation/Missed: ' + gtg_P + ' ' + m_gtg_P +
              '\nPercentage of Genes starting with TTG - Annotation/Missed: ' + ttg_P + ' ' + m_ttg_P +
              '\nPercentage of Genes starting with ATT - Annotation/Missed: ' + att_P + ' ' + m_att_P +
              '\nPercentage of Genes starting with CTG - Annotation/Missed: ' + ctg_P + ' ' + m_ctg_P +
              '\nPercentage of Genes starting with Alternative Start Codon - Annotation/Missed: ' + other_Starts_P + ' ' + m_other_Stops_P +
              '\nPercentage of Genes ending with TAG - Annotation/Missed: ' + tag_P + ' ' + m_tag_P +
              '\nPercentage of Genes ending with TAA - Annotation/Missed: ' + taa_P + ' ' + m_taa_P +
              '\nPercentage of Genes ending with TGA - Annotation/Missed: ' + tga_P + ' ' + m_tga_P +
              '\nPercentage of Genes ending with Alternative Stop Codon - Annotation/Missed: ' + other_Stops_P + ' ' + m_other_Stops_P)

    print(output)


if __name__ == "__main__":
    options = parser.parse_args()
    parameters = options.parameters
    tool = options.tool
    genome = options.genome
    if parameters:
        results_file = open('../Tools/' + tool + '/' + tool + '_' + genome + '_' + parameters + '.csv')
    else:
        results_file = open('../Tools/' + tool + '/' + tool + '_' + genome + '.csv')
    result_compare(genome, results_file)



#!/usr/bin/env python3
import collections

# Constants
SHORT_ORF_LENGTH = 300
MIN_COVERAGE = 75
ORForise_Version = 'v1.2.2'


def revCompIterative(watson):  # Gets Reverse Complement
    complements = {'A': 'T', 'T': 'A', 'C': 'G', 'G': 'C', 'N': 'N',
                   'R': 'Y', 'Y': 'R', 'S': 'S', 'W': 'W', 'K': 'M',
                   'M': 'K', 'V': 'B', 'B': 'V', 'H': 'D', 'D': 'H'}
    watson = watson.upper()
    watsonrev = watson[::-1]
    crick = ""
    for nt in watsonrev:
        try:
            crick += complements[nt]
        except KeyError:
            crick += nt  # Do not modify non-standard DNA
    return crick


def sortORFs(tool_ORFs):  # Will only sort by given start position
    tool_ORFs_Sorted = sorted(tool_ORFs.items(), key=lambda v: int(v[0].split(",")[0]))
    tool_ORFs_Sorted = collections.OrderedDict(tool_ORFs_Sorted)
    return tool_ORFs_Sorted


def sortGenes(Genes):  # Will sort by given start position and then rearrange for given stop
    Genes_Sorted_list = sorted(Genes.values(), key=lambda v: int(v[0]))
    Genes_Sorted = []
    for idx,gene in enumerate(Genes_Sorted_list):
        Genes_Sorted.append([idx,gene])
    Genes_Sorted = collections.OrderedDict(Genes_Sorted)
    prev_stop = 0
    for pos, detail in Genes_Sorted.items():
        if detail[1] < prev_stop:
            Genes_Sorted[pos], Genes_Sorted[pos-1] = Genes_Sorted[pos-1], Genes_Sorted[pos]
        prev_stop = detail[1]
    return Genes_Sorted


import collections

try:
    from utils import revCompIterative
    from utils import sortORFs
except ImportError:
    from ORForise.utils import revCompIterative
    from ORForise.utils import sortORFs


def GLIMMER_3(tool_pred, genome):
    GLIMMER_ORFs = collections.OrderedDict()
    genome_size = len(genome)
    genome_rev = revCompIterative(genome)
    with open(tool_pred,
              'r') as glimmer_input:  # GLIMMER_3 reverses the start and stop positions for ORFS on the negative strand
        for line in glimmer_input:
            if '>' not in line:  # This will not work with multiple contigs
                line = line.split()
                if len(line) == 5 and "orf" in line[0]:
                    if '-' in line[3]:  # Reverse Compliment starts and stops adjusted -  Switched to match Sense Strand
                        start = int(line[2])
                        stop = int(line[1])
                        strand = '-'
                        r_start = genome_size - stop
                        r_stop = genome_size - start
                        startCodon = genome_rev[r_start:r_start + 3]
                        stopCodon = genome_rev[r_stop - 2:r_stop + 1]
                    elif '+' in line[3]:
                        start = int(line[1])
                        stop = int(line[2])
                        strand = '+'
                        startCodon = genome[start - 1:start + 3]
                        stopCodon = genome[stop - 3:stop]
                    po = str(start) + ',' + str(stop)
                    orf = [strand, startCodon, stopCodon, 'CDS']
                    GLIMMER_ORFs.update({po: orf})

    GLIMMER_ORFs = sortORFs(GLIMMER_ORFs)
    return GLIMMER_ORFs



from importlib import import_module

import argparse
import collections
import csv

from Comparator import tool_comparison

###################


def comparator(tool, input_to_analyse, storfs_to_find_missing, genome_to_compare):
    genome_Seq = ""
    with open('Genomes/' + genome_to_compare + '.fa', 'r') as genome:
        for line in genome:
            line = line.replace("\n", "")
            if ">" not in line:
                genome_Seq += str(line)
    ##############################################
    genes = collections.OrderedDict()
    count = 0
    with open('Tools/StORF_Undetected/' + input_to_analyse, 'r') as genome_gff:  # Get list of missed genes
        for line in genome_gff:
            if ">" in line:
                line = line.strip()
                Start = int(line.split('_')[1])
                Stop = int(line.split('_')[2])
                Strand = line.split('_')[3]
                Gene = str(Start) + ',' + str(Stop) + ',' + Strand
                genes.update({count: Gene})
                count += 1
    ##################################
    tool_predictions = import_module('Tools.' + tool + '.' + tool)
    tool_predictions = getattr(tool_predictions, tool)
    orfs = tool_predictions(storfs_to_find_missing, genome_Seq)
    all_Metrics, all_rep_Metrics, start_precision, stop_precision, other_starts, other_stops, missed_genes, unmatched_orfs, undetected_gene_metrics, unmatched_orf_metrics, gene_coverage_genome, multi_Matched_ORFs, partial_Hits = tool_comparison(
        genes, orfs, genome_Seq)
    outname = tool + '_' + genome_to_compare
    metric_description = list(all_Metrics.keys())
    metrics = list(all_Metrics.values())
    rep_metric_description = list(all_rep_Metrics.keys())
    rep_metrics = list(all_rep_Metrics.values())
    with open("Tools/" + tool + '/' + outname + '.csv', 'w', newline='\n',
              encoding='utf-8') as out_file:  # Clear write out of report
        tool_out = csv.writer(out_file, quoting=csv.QUOTE_NONE, escapechar=" ")
        tool_out.writerow(['Representative_Metrics:'])
        tool_out.writerow(rep_metric_description)
        tool_out.writerow(rep_metrics)
        tool_out.writerow(['All_Metrics:'])
        tool_out.writerow(metric_description)
        tool_out.writerow(metrics)
        tool_out.writerow(['CDS_Gene_Coverage_of_Genome:'])
        tool_out.writerow([gene_coverage_genome])
        tool_out.writerow(['Start_Position_Difference:'])
        tool_out.writerow(start_precision)
        tool_out.writerow(['Stop_Position_Difference:'])
        tool_out.writerow(stop_precision)
        tool_out.writerow(['Alternative_Starts_Predicted:'])
        tool_out.writerow(other_starts)
        tool_out.writerow(['Alternative_Stops_Predicted:'])
        tool_out.writerow(other_stops)
        tool_out.writerow(['Undetected_Gene_Metrics:'])
        tool_out.writerow([
                              'ATG_Start,GTG_Start,TTG_Start,ATT_Start,CTG_Start,Alternative_Start_Codon,TGA_Stop,TAA_Stop,TAG_Stop,Alternative_Stop_Codon,Median_Length,ORFs_on_Positive_Strand,ORFs_on_Negative_Strand'])
        tool_out.writerow(undetected_gene_metrics)
        tool_out.writerow(['Undetected_Genes:'])
        for key, value in missed_genes.items():
            key = key.split(',')
            id = ('>' + genome_to_compare + '_' + key[0] + '_' + key[1] + '_' + key[2])
            tool_out.writerow([id + '\n' + value])
        tool_out.writerow(['\nORFs_Without_Corresponding_Gene_In_Ensembl_Metrics:'])
        tool_out.writerow([
            'ATG_Start,GTG_Start,TTG_Start,ATT_Start,CTG_Start,Alternative_Start_Codon,TGA_Stop,TAA_Stop,TAG_Stop,Alternative_Stop_Codon,Median_Length,ORFs_on_Positive_Strand,ORFs_on_Negative_Strand'])
        tool_out.writerow(unmatched_orf_metrics)
        tool_out.writerow(['ORF_Without_Corresponding_Gene_in_Ensembl:'])
        for key, value in unmatched_orfs.items():
            key = key.split(',')
            id = ('>' + tool + '_' + key[0] + '_' + key[1] + '_' + key[2])
            tool_out.writerow([id + '\n' + value])
        tool_out.writerow(['\nORFs_Which_Detected_more_than_one_Gene:'])

        try:
            for key, value in multi_Matched_ORFs.items():
                key = key.split(',')
                value = value[1].split(',')
                multi = ('ORF:' + key[0] + '-' + key[1] + '_Gene:' + value[0] + '-' + value[1])
                tool_out.writerow([multi])
        except IndexError:
            pass

        tool_out.writerow(['\n\nPartial_Gene_Hits:'])
        for key, seqs in partial_Hits.items():
            key = key.split(';')
            gene_Seq = seqs[0]
            orf_Seq = seqs[1]
            partial = (key[0] + '\n' + gene_Seq + '\n' + key[1] + '\n' + orf_Seq + '\n')
            tool_out.writerow([partial])


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--tool', default='GFF', help='Which tool/format would you analyse with StORF-R?')
    parser.add_argument('-i', '--input_to_analyse', default='', help='Location of file containing missed genes')
    parser.add_argument('-stf', '--storfs_to_find_missing', default='', help='STORFs to find missing.')
    parser.add_argument('-g', '--genome_to_compare', default='', help='Which genome to analyse?')
    args = parser.parse_args()

    comparator(**vars(args))

if __name__ == "__main__":
    main()



import collections
import sys
try:
    from utils import revCompIterative
    from utils import sortORFs
except ImportError:
    from ORForise.utils import revCompIterative
    from ORForise.utils import sortORFs


def GFF(**kwargs):
    tool_pred, genome,types = list(kwargs.values())
    GFF_ORFs = collections.OrderedDict()
    genome_size = len(genome)
    genome_rev = revCompIterative(genome)
    with open(tool_pred, 'r') as gff_input:
        for line in gff_input:
            if '#' not in line:
                line = line.split('\t')
                gene_types = types.split(',')
                if any(gene_type == line[2] for gene_type in gene_types)and len(line) == 9:  # line[2] for normalrun
                    start = int(line[3])
                    stop = int(line[4])
                    strand = line[6]
                    name = line[8].split('Name=')[1].split(';')[0] # Issue with multiple records for each gene.
                    if '-' in strand:  # Reverse Compliment starts and stops adjusted
                        r_start = genome_size - stop
                        r_stop = genome_size - start
                        startCodon = genome_rev[r_start:r_start + 3]
                        stopCodon = genome_rev[r_stop - 2:r_stop + 1]
                    elif '+' in strand:
                        startCodon = genome[start - 1:start + 2]
                        stopCodon = genome[stop - 3:stop]
                    po = str(start) + ',' + str(stop)
                    orf = [strand, startCodon, stopCodon, line[2],name] # This needs to detect the type
                    GFF_ORFs.update({po: orf})
                # elif "CDS" in line[2]:
                #     sys.exit("SAS")

    GFF_ORFs = sortORFs(GFF_ORFs)
    return GFF_ORFs



import argparse
import collections

parser = argparse.ArgumentParser()
parser.add_argument('-mg', '--missing_genes', default='', help='Which set of genes to check?')
parser.add_argument('-g', '--genome_to_compare', required=True,
                    help='Which genome to analyse? Genome files have same prefix'
                         ' - .fa and .gff appended')
args = parser.parse_args()


def comparator(missing_genes, genome_to_compare):
    missed_genes = collections.OrderedDict()
    non_vitiated_genes = []
    with open('../' + missing_genes, 'r') as m_genes:
        for line in m_genes:
            if line.startswith('>'):
                line = line.split('_')
                start = int(line[1])
                stop = int(line[2])
                m_gene_Set = set(range(start, stop + 1))
                missed_genes.update({str(start) + '_' + str(stop): m_gene_Set})
                non_vitiated_genes.append(str(start) + '_' + str(stop))
    ##############################################

    with open('../../Prodigal/Prodigal_' + genome_to_compare + '.gff', 'r') as prodigal_input:
        for line in prodigal_input:
            line = line.split()
            if "Prodigal" in line[1] and "CDS" in line[2]:
                start = int(line[3])
                stop = int(line[4])
                pred_set = set(range(start, stop + 1))
                for missed, g_set in missed_genes.items():
                    overlap = len(pred_set.intersection(g_set))
                    if overlap > 50:
                        print(start)
                        try:
                            non_vitiated_genes.remove(missed)
                            print(missed)
                        except ValueError:
                            continue
    print(len(non_vitiated_genes))


if __name__ == "__main__":
    comparator(**vars(args))



import collections

try:
    from utils import revCompIterative
    from utils import sortORFs
except ImportError:
    from ORForise.utils import revCompIterative
    from ORForise.utils import sortORFs

def GeneMark(tool_pred, genome):
    geneMark_ORFs = collections.OrderedDict()
    genome_size = len(genome)
    genome_rev = revCompIterative(genome)
    prev_Start = 0
    prev_Stop = 0
    started = False
    with open(tool_pred, 'r') as GeneMark_input:
        for line in GeneMark_input:
            line = line.split()
            if len(line) == 7:
                started = True
                if 'direct' in line[2] or 'complement' in line[
                    2]:  # Strange Output requires strange code - We select the Longest ORF from each set
                    start = int(line[0])
                    stop = int(line[1])
                    strand = line[2]
                    if 'complement' in strand:  # Reverse Compliment starts and stops adjusted
                        if start != prev_Start:
                            r_start = genome_size - stop
                            r_stop = genome_size - start
                            strand = '-'
                            startCodon = genome_rev[r_start:r_start + 3]
                            stopCodon = genome_rev[r_stop - 2:r_stop + 1]
                            po = str(start) + ',' + str(stop)
                            orf = [strand, startCodon, stopCodon, 'CDS']
                            geneMark_ORFs.update({po: orf})
                    elif 'direct' in strand:
                        if stop != prev_Stop:
                            startCodon = genome[start - 1:start + 2]
                            stopCodon = genome[stop - 3:stop]
                            strand = '+'
                            po = str(start) + ',' + str(stop)
                            orf = [strand, startCodon, stopCodon, 'CDS']
                            geneMark_ORFs.update({po: orf})
                    prev_Start = start
                    prev_Stop = stop
            elif len(line) == 0 and started == True:
                prev_Stop = 0
                prev_Start = 0

    geneMark_ORFs = sortORFs(geneMark_ORFs)
    return geneMark_ORFs

############# This section can be used to select the ORF with highest probability score.
# with open('Tools/GeneMark/' + input_to_analyse, 'r') as GeneMark_input:
#     prob_score = 0
#     started = False
#
#     for line in GeneMark_input:
#         line = line.split()
#
#         if len(line) == 7:
#             if 'direct' in line[2] or 'complement' in line[2] and '....' not in line[6] : # Strange Output requires strange code
#                 started = True
#                 start = int(line[0])
#                 stop = int(line[1])
#                 score = float(line[5])
#                 strand = line[2]
#                 if 'complement' in strand:  # Reverse Compliment starts and stops to confirm to our definition
#                     if start != prev_Start:
#                         prob_score = score
#                         # Switched to match Sense Strand
#                         r_start = genome_size - stop
#                         r_stop = genome_size - start
#                         strand = '-'
#                         startCodon = genome_rev[r_start:r_start + 3]
#                         stopCodon = genome_rev[r_stop - 2:r_stop + 1]
#                         po = str(start) + ',' + str(stop)
#                         orf = [strand, startCodon, stopCodon]
#                     elif start == prev_Start and score > prob_score:
#                         # Switched to match Sense Strand
#                         prob_score = score
#                         r_start = genome_size - stop
#                         r_stop = genome_size - start
#                         strand = '-'
#                         startCodon = genome_rev[r_start:r_start + 3]
#                         stopCodon = genome_rev[r_stop - 2:r_stop + 1]
#                         po = str(start) + ',' + str(stop)
#                         orf = [strand, startCodon, stopCodon]
#                 elif 'direct' in strand:
#                     if stop != prev_Stop:
#                         prob_score = score
#                         startCodon = genome[start - 1:start - 1 + 3]
#                         stopCodon = genome[stop - 3:stop - 1 + 1]
#                         strand = '+'
#                         po = str(start) + ',' + str(stop)
#                         orf = [strand, startCodon, stopCodon]
#                     elif stop == prev_Stop and score > prob_score:
#                         prob_score = score
#                         startCodon = genome[start - 1:start - 1 + 3]
#                         stopCodon = genome[stop - 3:stop - 1 + 1]
#                         strand = '+'
#                         po = str(start) + ',' + str(stop)
#                         orf = [strand, startCodon, stopCodon]
#                 prev_Start = start
#                 prev_Stop = stop
#         elif len(line) == 0 and started == True:
#             prob_score = 0
#             prev_Start = 0
#             prev_Stop = 0
#             GeneMark_ORFs.update({po: orf})
#             po = ''
#             orf = []
# #Remove last empty dict
# del GeneMark_ORFs['']
# print(GeneMark_ORFs)
# return GeneMark_ORFs



import argparse
import collections



parser = argparse.ArgumentParser()
parser.add_argument('-ref', '--reference_annotation', required=True,
                    help='Which reference annotation file to use as reference?')
parser.add_argument('-tp', '--tool_prediction', required=True, help='Tool genome prediction file')
args = parser.parse_args()


def main(reference_annotation,tool_prediction):
    ref_genes = collections.OrderedDict()  # Order is important
    num_hypo = 0
    with open(reference_annotation, 'r') as genome_gff:
        for line in genome_gff:
            line = line.split('\t')
            try:
                if "gene" in line[2] and len(line) == 9: # Have to use gene and not CDS here because the CDS tag does not contain the classification
                    start = line[3]
                    stop = line[4]
                    gene = start+'_'+stop
                    if "hypothetical" in line[8]:
                        ref_genes.update({gene:"hypothetical"})
                        num_hypo +=1
                    else:
                        ref_genes.update({gene: "N/A"})
            except IndexError:
                continue
    print("Number of hypothetic genes: "+str(num_hypo))
    ###############################
    perfect_match_hypo, partial_match_hypo, missed_hypo = 0,0,0
    perfect_match = False
    partial_match = False
    missed = False
    posss = []
    with open(tool_prediction, 'r') as tool_in:
        for line in tool_in:
            if line.startswith("Perfect_Match_Genes:"):
                perfect_match = True
            elif line.startswith("Partial_Match_Genes:"):
                perfect_match = False
                partial_match = True
            elif line.startswith("Missed_Genes:"):
                perfect_match = False
                partial_match = False
                missed = True
            elif line.startswith("ORF_Without_Corresponding_Gene_in_Ensembl"):
                break
            #################
            if perfect_match == True:
                if line.startswith('>'):
                    pos = line.split('_')[1] +'_'+ line.split('_')[2]
                    try:
                        if "hypothetical" in ref_genes[pos]:
                            perfect_match_hypo +=1
                            print(pos)
                            if pos in posss:
                                print("WE")
                            posss.append(pos)
                    except KeyError:
                        continue
            elif partial_match == True:
                if line.startswith('Gene:'): # Different tags
                    pos = line.split('_')[0].split(':')[1] +'_'+ line.split('_')[1] # should change Orforise output
                    try:
                        if "hypothetical" in ref_genes[pos]:
                            partial_match_hypo +=1
                            print(pos)
                    except KeyError:
                        continue
            elif missed == True:
                if line.startswith('>'):
                    pos = line.split('_')[1] +'_'+ line.split('_')[2]
                    try:
                        if "hypothetical" in ref_genes[pos]:
                            missed_hypo +=1
                            print(pos)
                    except KeyError:
                        continue

    print("finished")

if __name__ == "__main__":
    main(**vars(args))

    print("Complete")



import collections

try:
    from utils import revCompIterative
    from utils import sortORFs
except ImportError:
    from ORForise.utils import revCompIterative
    from ORForise.utils import sortORFs


def StORF_Undetected(tool_pred, genome):
    storf_orfs = collections.OrderedDict()
    genome_size = len(genome)
    genome_Rev = revCompIterative(genome)
    with open(tool_pred, 'r') as storf_input:
        for line in storf_input:
            line = line.split()
            if "StORF" in line[1] and "ORF" in line[2]:
                start = int(line[3])
                stop = int(line[4])
                strand = line[6]
                if '-' in strand:  # Reverse Compliment starts and stops adjusted
                    r_start = genome_size - stop
                    r_stop = genome_size - start
                    startCodon = genome_Rev[r_start:r_start + 3]
                    stopCodon = genome_Rev[r_stop - 2:r_stop + 1]
                elif '+' in strand:
                    startCodon = genome[start - 1:start - 1 + 3]
                    stopCodon = genome[stop - 3:stop - 1 + 1]
                po = str(start) + ',' + str(stop)
                orf = [strand, startCodon, stopCodon]
                storf_orfs.update({po: orf})

    storf_orfs = sortORFs(storf_orfs)
    return storf_orfs



import copy

import argparse
import collections

from ORForise.src.ORForise.utils import *

parser = argparse.ArgumentParser()
parser.add_argument('-t', '--tool', required=True, help='Which tool to compare?')
parser.add_argument('-p', '--parameters', required=False, help='Optional parameters for prediction tool.')
parser.add_argument('-g', '--genome', required=True, help='Which genome to analyse?')
args = parser.parse_args()


def gc_count(dna):
    c = 0
    a = 0
    g = 0
    t = 0
    n = 0
    for i in dna:
        if "C" in i:
            c += 1
        elif "G" in i:
            g += 1
        elif "A" in i:
            a += 1
        elif "T" in i:
            t += 1
        elif "N" in i:
            n += 1
    gc_content = format((g + c) * 100 / (a + t + g + c + n), '.2f')
    n_per = n * 100 / (a + t + g + c + n)
    return n_per, gc_content


def start_Codon_Count(orfs):
    atg, gtg, ttg, att, ctg, other = 0, 0, 0, 0, 0, 0
    other_Starts = []
    for orf in orfs.values():
        codon = orf[-2]
        if codon == 'ATG':
            atg += 1
        elif codon == 'GTG':
            gtg += 1
        elif codon == 'TTG':
            ttg += 1
        elif codon == 'ATT':
            att += 1
        elif codon == 'CTG':
            ctg += 1
        else:
            other += 1
            other_Starts.append(codon)
    atg_P = format(100 * atg / len(orfs), '.2f')
    gtg_P = format(100 * gtg / len(orfs), '.2f')
    ttg_P = format(100 * ttg / len(orfs), '.2f')
    att_P = format(100 * att / len(orfs), '.2f')
    ctg_P = format(100 * ctg / len(orfs), '.2f')
    other_Start_P = format(100 * other / len(orfs), '.2f')
    return atg_P, gtg_P, ttg_P, att_P, ctg_P, other_Start_P, other_Starts


def stop_Codon_Count(orfs):
    tag, taa, tga, other = 0, 0, 0, 0
    other_Stops = []
    for orf in orfs.values():
        codon = orf[-1]
        if codon == 'TAG':
            tag += 1
        elif codon == 'TAA':
            taa += 1
        elif codon == 'TGA':
            tga += 1
        else:
            other += 1
            other_Stops.append(codon)
    tag_p = format(100 * tag / len(orfs), '.2f')
    taa_p = format(100 * taa / len(orfs), '.2f')
    tga_p = format(100 * tga / len(orfs), '.2f')
    other_Stop_P = format(100 * other / len(orfs), '.2f')
    return tag_p, taa_p, tga_p, other_Stop_P, other_Stops


def detail_transfer(genes, missed_genes):
    for missed, m_details in missed_genes.items():
        try:
            details = genes[missed]
            gc = details[2]
            up_Overlap = details[3]
            down_Overlap = details[4]
            m_details.insert(2, gc)
            m_details.insert(3, up_Overlap)
            m_details.insert(4, down_Overlap)
        except KeyError:
            pass
    return missed_genes


def get_genome(genome):
    genome_Seq = ""
    with open('../Genomes/' + genome + '.fa', 'r') as genome:
        for line in genome:
            line = line.replace("\n", "")
            if not line.startswith('>'):
                genome_Seq += str(line)
    return genome_Seq


def missed_genes_in(genes_detected, missed_genes, results_in):
    # Missed Genes Read-In
    read = False
    for line in results_in:
        line = line.strip()
        if read == True:
            if line.startswith('>'):
                entry = line.split('_')
                entry = entry[1] + '_' + entry[2]
                strand = entry[-1]
                if int(strand) <= 2:
                    strand = '+'
                else:
                    strand = '-'
            elif len(line.strip()) > 0:
                startCodon = line[0:3]
                stopCodon = line[-3:]
                length = len(line)
                missed_genes.update({entry: [line, strand, length, startCodon, stopCodon]})

        if line.startswith('Undetected_Genes:'):
            read = True
        if read == True and not line:
            break
    list_Missed = list(missed_genes.keys())
    for key in list_Missed:
        # ### printed out to confirm figure lengths
        # start = key.split('_')[0]
        # stop = key.split('_')[1]
        if key in genes_detected:
            del genes_detected[key]

    return missed_genes, genes_detected


def partial_matches_in(partial_matches, results_in):
    # partial Genes Read-In
    read = False
    prev = ''
    for line in results_in:
        line = line.strip()
        if read == True:
            if line.startswith('Gene:'):
                line = line.replace('Gene:', '')
                entry = line.split('_')
                g_Pos = entry[0] + '_' + entry[1]
                strand = entry[2]
                prev = 'Gene'
            elif line.startswith('ORF:'):
                line = line.replace('ORF:', '')
                entry = line.split('_')
                o_Pos = entry[0] + '_' + entry[1]
                prev = 'ORF'
            elif line:
                if 'Gene' in prev:
                    g_Seq = line.strip()
                    g_length = len(g_Seq)
                elif 'ORF' in prev:
                    o_Seq = line.strip()
                    orf_length = len(o_Seq)
            elif not line:
                partial_matches.update({g_Pos: [strand, g_length, g_Seq, o_Pos, orf_length, o_Seq]})
        if line.startswith('Partial_Gene_Hits:'):
            read = True

    return partial_matches


def unmatched_ORFs_in(unmatched_ORFs, results_file):
    # Unmatched ORFs Read-In
    read = False
    for line in results_file:
        line = line.strip()
        if read == True:
            if line.startswith('>'):
                line = line.replace('Gene:', '')
                entry = line.split('_')
                strand = entry[-1]
                o_Pos = entry[1] + '_' + entry[2]
                unmatched_ORFs.update({o_Pos: [strand, None, None, None, None]})
            elif line:
                o_Seq = line.strip()
                o_Length = len(o_Seq)
                startCodon = line[0:3]
                stopCodon = line[-3:]
                unmatched_ORFs.update({o_Pos: [strand, o_Length, o_Seq, startCodon, stopCodon]})
        if line.startswith('ORF_Without_Corresponding_Gene_in_Ensembl:'):
            read = True
        elif read == True and not line:
            unmatched_ORFs.update({o_Pos: [strand, o_Length, o_Seq, startCodon, stopCodon]})
            break

    return unmatched_ORFs


def genes_in(genome, genome_Seq, genome_Seq_Rev, genome_Size, genes):
    with open('../Genomes/' + genome + '.gff', 'r') as genome_gff:
        for line in genome_gff:
            line = line.split('\t')
            try:
                if "CDS" in line[2] and len(line) == 9:
                    start = int(line[3])
                    stop = int(line[4])
                    strand = line[6]
                    length = stop - start
                    gene = str(start) + '_' + str(stop)
                    if '+' in strand:
                        seq = genome_Seq[start - 1:stop]
                    elif '-' in strand:
                        r_Start = genome_Size - stop
                        r_Stop = genome_Size - start
                        seq = genome_Seq_Rev[r_Start:r_Stop + 1]
                    startCodon = seq[0:3]
                    stopCodon = seq[-3:]
                    genes.update({gene: [seq, strand, length, startCodon, stopCodon]})
            except IndexError:
                continue
    return genes


def extract_results(genome, results_file):
    genome_Seq = get_genome(genome)
    genome_Seq_Rev = revCompIterative(genome_Seq)
    genome_Size = len(genome_Seq)
    genes = collections.OrderedDict()
    partial_matches = collections.OrderedDict()
    missed_genes = collections.OrderedDict()
    unmatched_ORFs = collections.OrderedDict()

    genes = genes_in(genome, genome_Seq, genome_Seq_Rev, genome_Size, genes)
    genes_detected = copy.deepcopy(genes)
    missed_genes, genes_detected = missed_genes_in(genes_detected, missed_genes, results_file)
    results_file.seek(0, 0)  # Reset file position
    partial_matches = partial_matches_in(partial_matches, results_file)
    results_file.seek(0, 0)  # Reset file position
    unmatched_ORFs = unmatched_ORFs_in(unmatched_ORFs, results_file)
    results_file.seek(0, 0)  # Reset file position

    return genes, genes_detected, missed_genes, partial_matches, unmatched_ORFs


if __name__ == "__main__":
    options = parser.parse_args()
    parameters = options.parameters
    tool = options.tool
    genome = options.genome
    if parameters:
        results_file = open('../Tools/' + tool + '/' + tool + '_' + genome + '_' + parameters + '.csv')
    else:
        results_file = open('../Tools/' + tool + '/' + tool + '_' + genome + '.csv')

    genes, genes_detected, missed_genes, partial_matches, unmatched_ORFs = extract_results(genome, results_file)
    gene_Lengths, genes_detected_Lengths, missed_Lengths, partial_Lengths, unmatched_Lengths = [], [], [], [], []

    for pos in genes.keys():
        gene_Lengths.append(int(pos.split('_')[1]) - int(pos.split('_')[0]))
    for pos in genes_detected.keys():
        genes_detected_Lengths.append(int(pos.split('_')[1]) - int(pos.split('_')[0]))
    for pos in partial_matches.keys():
        partial_Lengths.append(int(pos.split('_')[1]) - int(pos.split('_')[0]))
    for pos in missed_genes.keys():
        missed_Lengths.append(int(pos.split('_')[1]) - int(pos.split('_')[0]))
    for pos in unmatched_ORFs.keys():
        unmatched_Lengths.append(int(pos.split('_')[1]) - int(pos.split('_')[0]))

    import numpy as np

    print(len(gene_Lengths))
    print(gene_Lengths)
    print(np.mean(gene_Lengths))
    print(len(partial_Lengths))
    print(partial_Lengths)
    print(len(missed_Lengths))
    print(missed_Lengths)
    print(len(unmatched_Lengths))
    print(unmatched_Lengths)
    print(np.mean(unmatched_Lengths))



import collections

try:
    from utils import revCompIterative
    from utils import sortORFs
except ImportError:
    from ORForise.utils import revCompIterative
    from ORForise.utils import sortORFs


def StORF_Reporter(tool_pred, genome):
    storf_orfs = collections.OrderedDict()
    genome_size = len(genome)
    genome_rev = revCompIterative(genome)
    with open(tool_pred, 'r') as storf_input:
        for line in storf_input:
            if '#' not in line:
                line = line.split()
                if 'StORF_Reporter' in line[1] or 'StoRF_Reporter' in line[1]: # need to harmonise this.
                    start = int(line[3])
                    stop = int(line[4])
                    strand = line[6]
                    if '-' in strand:  # Reverse Compliment starts and stops adjusted
                        r_start = genome_size - stop
                        r_stop = genome_size - start
                        startCodon = genome_rev[r_start:r_start + 3]
                        stopCodon = genome_rev[r_stop - 2:r_stop + 1]
                    elif '+' in strand:
                        startCodon = genome[start:start + 3]
                        stopCodon = genome[stop - 3:stop]
                    po = str(start) + ',' + str(stop)
                    orf = [strand, startCodon, stopCodon, line[2]] # StORF/Con-StORF or CDS??
                    storf_orfs.update({po: orf})

    storf_orfs = sortORFs(storf_orfs)
    return storf_orfs



import argparse
import collections
import numpy as np
import sys

parser = argparse.ArgumentParser()
parser.add_argument('-t', '--tool', required=True, help='Which tool to compare?')
parser.add_argument('-p', '--parameters', required=False, help='Optional parameters for prediction tool.')
parser.add_argument('-g', '--genome', required=True, help='Which genome to analyse?')

args = parser.parse_args()


def gc_count(dna):
    c = 0
    a = 0
    g = 0
    t = 0
    n = 0
    for i in dna:
        if "C" in i:
            c += 1
        elif "G" in i:
            g += 1
        elif "A" in i:
            a += 1
        elif "T" in i:
            t += 1
        elif "N" in i:
            n += 1
    gc_content = format((g + c) * 100 / (a + t + g + c + n), '.2f')
    return gc_content


def revCompIterative(watson):
    complements = {'A': 'T', 'T': 'A', 'C': 'G', 'G': 'C', 'N': 'N'}
    watson = watson.upper()
    watsonrev = watson[::-1]
    crick = ""
    for nt in watsonrev:
        crick += complements[nt]
    return crick


def start_Codon_Count(starts, lengths):
    try:
        atg_P = format(100 * starts['ATG'] / len(lengths), '.2f')
        gtg_P = format(100 * starts['GTG'] / len(lengths), '.2f')
        ttg_P = format(100 * starts['TTG'] / len(lengths), '.2f')
        att_P = format(100 * starts['ATT'] / len(lengths), '.2f')
        ctg_P = format(100 * starts['CTG'] / len(lengths), '.2f')
    except ZeroDivisionError:
        atg_P, gtg_P, ttg_P, att_P, ctg_P = 0, 0, 0, 0, 0
    return atg_P, gtg_P, ttg_P, att_P, ctg_P  # ,other_Start_P,other_Starts


def stop_Codon_Count(stops, lengths):
    try:
        tag_P = format(100 * stops['TAG'] / len(lengths), '.2f')
        taa_P = format(100 * stops['TAA'] / len(lengths), '.2f')
        tga_P = format(100 * stops['TGA'] / len(lengths), '.2f')
    # return atg_P, gtg_P, ttg_P, att_P, ctg_P  # ,other_Start_P,other_Starts
    except ZeroDivisionError:
        tag_P, taa_P, tga_P = 0, 0, 0
    return tag_P, taa_P, tga_P  # ,other_Stop_P,other_Stops


def partial_gene_read(results_file, partial_genes):
    # partial Genes Read-In
    orf_Lengths = []
    gene_Lengths = []

    read = False
    prev = ''
    for line in results_file:
        line = line.strip()
        if read == True:
            if line.startswith('Gene:'):
                line = line.replace('Gene:', '')
                entry = line.split('_')
                g_Pos = entry[0] + '_' + entry[1]
                strand = entry[2]
                prev = 'Gene'
            elif line.startswith('ORF:'):
                line = line.replace('ORF:', '')
                entry = line.split('_')
                o_Pos = entry[0] + '_' + entry[1]
                prev = 'ORF'
            elif line:
                if 'Gene' in prev:
                    g_Seq = line.strip()
                    gene_Lengths.append(len(g_Seq))
                elif 'ORF' in prev:
                    o_Seq = line.strip()
                    orf_Lengths.append(len(o_Seq))
            elif not line:
                g_GC = gc_count(g_Seq)
                o_GC = gc_count(o_Seq)
                partial_genes.update({g_Pos: [strand, g_Seq, g_GC, o_Pos, o_Seq, o_GC]})
        if line.startswith('Partial_Gene_Hits:'):
            read = True

    return partial_genes, orf_Lengths, gene_Lengths


def detail_transfer(genes, partial_genes):
    for partial, m_details in partial_genes.items():
        try:
            details = genes[partial]
            gc = details[2]
            up_Overlap = details[3]
            down_Overlap = details[4]
            m_details.insert(2, gc)
            m_details.insert(3, up_Overlap)
            m_details.insert(4, down_Overlap)
        except KeyError:
            pass
    return partial_genes


def result_compare(genome, results_file):
    genome_Seq = ""
    with open('../Genomes/' + genome + '.fa', 'r') as genome_file:
        for line in genome_file:
            line = line.replace("\n", "")
            if ">" not in line:
                genome_Seq += str(line)

    partial_genes = collections.OrderedDict()
    partial_genes, orf_Lengths, gene_Lengths = partial_gene_read(results_file, partial_genes)
    orf_Median = np.median(orf_Lengths)
    gene_Median = np.median(gene_Lengths)
    strands = collections.defaultdict(int, {'-': 0, '+': 0})
    # Hard coded codons - Not ideal - default dicts?
    gene_Starts = collections.OrderedDict(
        {'ATG': 0, 'ATT': 0, 'CTG': 0, 'GAC': 0, 'GTG': 0, 'TTG': 0, 'ATC': 0, 'ATA': 0})
    gene_Stops = collections.OrderedDict({'TAA': 0, 'TAG': 0, 'TGA': 0})
    gene_GC = []
    orf_Starts = collections.OrderedDict(
        {'ATG': 0, 'ATT': 0, 'CTG': 0, 'GAC': 0, 'GTG': 0, 'TTG': 0, 'ATC': 0, 'ATA': 0})
    orf_Stops = collections.OrderedDict({'TAA': 0, 'TAG': 0, 'TGA': 0})
    orf_GC = []

    for gene, data in partial_genes.items():
        print(
            "\nPartial Matched Gene:\t" + gene + "\t" + data[1] + "\nPartial Matched ORF:\t" + data[3] + "\t" + data[4])
        strands[data[0]] += 1
        try:
            gene_Starts[data[1][0:3]] += 1
            gene_Stops[data[1][-3:]] += 1
            gene_GC.append(float(data[2]))
            orf_Starts[data[4][0:3]] += 1
            orf_Stops[data[4][-3:]] += 1
            orf_GC.append(float(data[5]))
        except KeyError:
            sys.exit("Key Error: " + str(data))

    gene_Median_GC = np.median(gene_GC)
    orf_Median_GC = np.median(orf_GC)
    # atg_P = format(100* gene_Starts['ATG'] / len(gene_Lengths),'.2f')
    # gtg_P = format(100 * gene_Starts['GTG'] / len(gene_Lengths),'.2f')
    # ttg_P = format(100 * gene_Starts['TTG'] / len(gene_Lengths),'.2f')
    # att_P = format(100 * gene_Starts['ATT']  / len(gene_Lengths),'.2f')
    # ctg_P = format(100 * gene_Starts['CTG']  / len(gene_Lengths),'.2f')
    # #other_Start_P = format(100 * other / len(gene_Lengths),'.2f')
    #
    # orf_GC_Median = format(np.median(pcg_GC),'.2f')
    # num_Short_PCGs = len(short_PCGs)
    #
    # partial_genes = detail_transfer(genes,partial_genes)

    g_atg_P, g_gtg_P, g_ttg_P, g_att_P, g_ctg_P = start_Codon_Count(gene_Starts, gene_Lengths)
    g_tag_P, g_taa_P, g_tga_P = stop_Codon_Count(gene_Stops, gene_Lengths)
    o_atg_P, o_gtg_P, o_ttg_P, o_att_P, o_ctg_P = start_Codon_Count(orf_Starts, orf_Lengths)
    o_tag_P, o_taa_P, o_tga_P = stop_Codon_Count(orf_Stops, orf_Lengths)

    # output = ("Number of Protein Coding Genes in " + str(annotation) + " : " + str(len(gene_Lengths)) + ", Median Length of PCGs: " +
    #           str(gene_Median) + ", Min Length of PCGs: " + str('NA') + ", Max Length of PCGs: " + str('NA') +
    #           ", Number of PCGs on Pos Strand: " + str(strands['+']) + ", Number of PCGs on Neg Strand: " + str(strands['-']) +
    #           ", Median GC of PCGs: " + str('NA') +
    #           ", Number of PCGs less than 100nt: " + str('NA') +
    output = ("Number of Partial Hits:" + str(len(gene_Lengths)) + "\nMedian Length of Partial Hit Genes:" + str(
        gene_Median) +
              '\nMedian Length of Partial Hit ORFs:' + str(orf_Median) + '\nMedian GC Partial Hit Genes:' + str(
                gene_Median_GC) +
              '\nMedian GC Partial Hit ORFs:' + str(orf_Median_GC) +
              '\nPercentage of Genes starting with ATG - Annotation/partial: ' + g_atg_P + ' ' + o_atg_P +
              '\nPercentage of Genes starting with GTG - Annotation/partial: ' + g_gtg_P + ' ' + o_gtg_P +
              '\nPercentage of Genes starting with TTG - Annotation/partial: ' + g_ttg_P + ' ' + o_ttg_P +
              '\nPercentage of Genes starting with ATT - Annotation/partial: ' + g_att_P + ' ' + o_att_P +
              '\nPercentage of Genes starting with CTG - Annotation/partial: ' + g_ctg_P + ' ' + o_ctg_P +
              # '\nPercentage of Genes starting with Alternative Start Codon - Annotation/partial: ' + other_Starts_P + ' ' + m_other_Stops_P +
              '\nPercentage of Genes ending with TAG - Annotation/partial: ' + g_tag_P + ' ' + o_tag_P +
              '\nPercentage of Genes ending with TAA - Annotation/partial: ' + g_taa_P + ' ' + o_taa_P +
              '\nPercentage of Genes ending with TGA - Annotation/partial: ' + g_tga_P + ' ' + o_tga_P)
    # '\nPercentage of Genes ending with Alternative Stop Codon - Annotation/partial: ' + other_Stops_P + ' ' + m_other_Stops_P)

    print(output)

    # import matplotlib.pylab as plt
    #
    # list_ORF_Starts = list(orf_Starts.items())  # sorted by key, return a list of tuples
    # list_Gene_Starts = list(gene_Starts.items())
    # o_x, o_y = zip(*list_ORF_Starts)  # unpack a list of pairs into two tuples
    # g_x, g_y = zip(*list_Gene_Starts)
    #
    # plt.plot(o_x, o_y)
    # plt.plot(g_x, g_y)
    # plt.show()
    #
    # list_ORF_Stops = list(orf_Stops.items())  # sorted by key, return a list of tuples
    # list_Gene_Stops = list(gene_Stops.items())
    # o_x, o_y = zip(*list_ORF_Stops)  # unpack a list of pairs into two tuples
    # g_x, g_y = zip(*list_Gene_Stops)
    #
    # plt.plot(o_x, o_y)
    # plt.plot(g_x, g_y)
    # plt.show()


if __name__ == "__main__":
    options = parser.parse_args()
    parameters = options.parameters
    tool = options.tool
    genome = options.genome
    if parameters:
        results_file = open('../Tools/' + tool + '/' + tool + '_' + genome + '_' + parameters + '.csv')
    else:
        results_file = open('../Tools/' + tool + '/' + tool + '_' + genome + '.csv')
    result_compare(genome, results_file)



from importlib import import_module
import argparse
import collections
from datetime import date
import sys
try:
    from utils import *
except ImportError:
    from .utils import *

################################


def gff_writer(genome_ID, genome_DNA,reference_annotation, reference_tool, ref_gene_set, additional_annotation, additional_tool, genes_To_Keep, output_file):
    write_out = open(output_file, 'w')
    write_out.write("##gff-version\t3\n#\tGFF-Intersector\n#\tRun Date:" + str(date.today()) + '\n')
    write_out.write("##Genome DNA File:" + genome_DNA + '\n')
    write_out.write("##Original File: " + reference_annotation + "\n##Intersecting File: " + additional_annotation + '\n')
    for pos, data in genes_To_Keep.items():
        pos_ = pos.split(',')
        start = pos_[0]
        stop = pos_[-1]
        strand = data[0]
        type = 'original'
        entry = (
                    genome_ID + '\t' + type + '\t' + data[2] + '\t' + start + '\t' + stop + '\t.\t' + strand + '\t.\tID=Original_Annotation;Coverage=' + str(
                data[1]) + '\n')
        write_out.write(entry)


def comparator(options):  # Only works for single contig genome
    genome_seq = ""
    with open(options.genome_DNA, 'r') as genome_fasta:
        for line in genome_fasta:
            line = line.replace("\n", "")
            if not line.startswith('>'):
                genome_seq += str(line)
            else:
                genome_ID = line.split()[0].replace('>', '')
    ###########################################
    if not options.reference_tool:  # IF using Ensembl for comparison
        ref_genes = collections.OrderedDict()  # Order is important
        count = 0
        with open(options.reference_annotation, 'r') as genome_gff:
            for line in genome_gff:
                line = line.split('\t')
                try:
                    if 'CDS' in options.gene_ident and len(options.gene_ident) == 1:
                        if "CDS" in line[2] and len(line) == 9:
                            start = int(line[3])
                            stop = int(line[4])
                            strand = line[6]
                            pos = str(start) + ',' + str(stop)
                            ref_genes.update({pos: [strand, 'ref', 'CDS']})
                            count += 1
                    else:
                        gene_types = options.gene_ident.split(',')
                        if any(gene_type in line[2] for gene_type in gene_types):  # line[2] for normalrun
                            start = int(line[3])
                            stop = int(line[4])
                            strand = line[6]
                            pos = str(start) + ',' + str(stop)
                            ref_genes.update(
                                {pos: [strand, 'ref', line[2]]})  # Report what type of gene/rRNA etc we have here
                            count += 1
                except IndexError:
                    continue
    else:  # IF using a tool as reference
        try:
            reference_tool_ = import_module('Tools.' + options.reference_tool + '.' + options.reference_tool,
                                             package='my_current_pkg')
        except ModuleNotFoundError:
            try:
                reference_tool_ = import_module('ORForise.Tools.' + options.reference_tool + '.' + options.reference_tool,
                                             package='my_current_pkg')
            except ModuleNotFoundError:
                sys.exit("Tool not available")
        reference_tool_ = getattr(reference_tool_, options.reference_tool)
        ############ Reformatting tool output for ref_genes
        ref_genes = reference_tool_(options.reference_annotation, genome_seq)
    ref_gene_set = list(ref_genes.keys())
    ############################## Get Add'l
    try:
        additional_tool_ = import_module('Tools.' + options.additional_tool + '.' + options.additional_tool,
                                        package='my_current_pkg')
    except ModuleNotFoundError:
        try:
            additional_tool_ = import_module('ORForise.Tools.' + options.additional_tool + '.' + options.additional_tool,
                                            package='my_current_pkg')
        except ModuleNotFoundError:
            sys.exit("Tool not available")
    additional_tool_ = getattr(additional_tool_, options.additional_tool)
    additional_orfs = additional_tool_(options.additional_annotation, genome_seq)
    ##############################


    genes_To_Keep = collections.OrderedDict()

    if options.coverage == 100:
        for orf, data in additional_orfs.items():
            o_Start = int(orf.split(',')[0])
            o_Stop = int(orf.split(',')[1])
            o_Strand = data[0]
            try:
                if ref_genes[str(o_Start) + ',' + str(o_Stop)][2] == "CDS" : # Make sure 100% match and is also CDS
                    genes_To_Keep.update({str(o_Start) + ',' + str(o_Stop): [o_Strand, options.coverage,"CDS"]})  # o_ and g_ would be the same here
            except KeyError:
                continue
    else:
        for orf, data in additional_orfs.items():  # Currently allows ORF to be bigger than Gene
            o_Start = int(orf.split(',')[0])
            o_Stop = int(orf.split(',')[1])
            o_Strand = data[0]
            orf_Set = set(range(int(o_Start), int(o_Stop) + 1))
            for gene, g_data in ref_genes.items():  # Very ineffecient
                g_Start = int(gene.split(',')[0])
                g_Stop = int(gene.split(',')[1])
                g_Strand = g_data[0]
                gene_Set = set(range(int(g_Start), int(g_Stop) + 1))
                overlap = len(orf_Set.intersection(gene_Set))
                cov = 100 * float(overlap) / float(len(gene_Set))
                if abs(o_Stop - g_Stop) % 3 == 0 and o_Strand == g_Strand and cov >= options.coverage:
                    genes_To_Keep.update({str(g_Start) + ',' + str(g_Stop): [g_Strand, int(cov),g_data[2]]})
                if g_Start > o_Stop:
                    break
    #########################################################
    #### Currently, only CDSs are filtered
    for gene, g_data in ref_genes.items():  # Very ineffecient
        if "CDS" not in g_data[2]:
            g_Start = int(gene.split(',')[0])
            g_Stop = int(gene.split(',')[1])
            g_Strand = g_data[0]
            genes_To_Keep.update({str(g_Start) + ',' + str(g_Stop): [g_Strand, "N/A",g_data[2]]})
    genes_To_Keep = sortORFs(genes_To_Keep)
    gff_writer(genome_ID, options.genome_DNA, options.reference_annotation, options.reference_tool, ref_gene_set, options.additional_annotation, options.additional_tool, genes_To_Keep, options.output_file)

def main():
    print("Thank you for using ORForise\nPlease report any issues to: https://github.com/NickJD/ORForise/issues\n#####")

    parser = argparse.ArgumentParser(description='ORForise ' + ORForise_Version + ': GFF-Intersector Run Parameters.')
    parser._action_groups.pop()

    required = parser.add_argument_group('Required Arguments')
    required.add_argument('-dna', dest='genome_DNA', required=True, help='Genome DNA file (.fa) which both annotations '
                                                                    'are based on')
    required.add_argument('-ref', dest='reference_annotation', required=True,
                        help='Which reference annotation file to use as reference?')
    required.add_argument('-at', dest='additional_tool', required=True,
                        help='Which format to use for additional annotation?')
    required.add_argument('-add', dest='additional_annotation', required=True,
                        help='Which annotation file to add to reference annotation?')
    required.add_argument('-o', dest='output_file', required=True,
                        help='Output filename')

    optional = parser.add_argument_group('Optional Arguments')
    optional.add_argument('-rt', dest='reference_tool', required=False,
                        help='Which tool format to use as reference? - If not provided, will default to '
                             'standard Ensembl GFF format, can be Prodigal or any of the other tools available')
    optional.add_argument('-gi', dest='gene_ident', default='CDS', required=False,
                        help='Identifier used for extraction of "genic" regions from reference annotation '
                             '"CDS,rRNA,tRNA": Default for is "CDS"')
    optional.add_argument('-cov', dest='coverage', default=100, type=int, required=False,
                        help='Percentage coverage of reference annotation needed to confirm intersection'
                             ' - Default: 100 == exact match')

    options = parser.parse_args()
    comparator(options)



if __name__ == "__main__":
    main()
    print("Complete")



import argparse
import numpy as np

parser = argparse.ArgumentParser()
parser.add_argument('-g', '--genome_to_compare', default='', help='Which genome to analyse?')
args = parser.parse_args()


def genome_Lengths(genome_to_compare):
    lengths = []
    with open('../Genomes/' + genome_to_compare + '.gff', 'r') as genome_gff:
        for line in genome_gff:
            line = line.split('\t')
            try:
                if "CDS" in line[2] and len(line) == 9:
                    start = int(line[3])
                    stop = int(line[4])
                    length = stop - start
                    lengths.append(length)
            except IndexError:
                # print(line)
                continue
    print("Number of Genes: " + str(len(lengths)) +
          '\tMedian Length of Genes: ' + str(np.median(lengths)) + '\nGenes Lengths:\n' + str(lengths))


if __name__ == "__main__":
    genome_Lengths(**vars(args))
